@@ -1,5 +1,5 @@
 /*
- * Remmina - The GTK+ Remote Desktop Client
+ * Remmina - The GTK Remote Desktop Client
  * Copyright (C) 2010 Jay Sorg
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2011 Marc-Andre Moreau <marcandre.moreau@gmail.com>
@@ -43,7 +43,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <cairo/cairo-xlib.h>
 #include <freerdp/locale/keyboard.h>
-#include <execinfo.h>
 
 static void remmina_rdp_event_on_focus_in(GtkWidget* widget, GdkEventKey* event, RemminaProtocolWidget* gp)
 {
@@ -291,13 +290,13 @@ static gboolean remmina_rdp_event_on_draw(GtkWidget* widget, cairo_t* context, R
 
 
 	if (rfi->is_reconnecting) {
-		/* freerdp is reconnecting, just show a message to the user */
+		/* FreeRDP is reconnecting, just show a message to the user */
 
 		width = gtk_widget_get_allocated_width(widget);
 		height = gtk_widget_get_allocated_height(widget);
 
 		/* Draw text */
-		msg = g_strdup_printf(_("Reconnection in progress. Attempt %d of %d…"),
+		msg = g_strdup_printf(_("Reconnection attempt %d of %d…"),
 			rfi->reconnect_nattempt, rfi->reconnect_maxattempts);
 
 		cairo_select_font_face(context, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
@@ -309,7 +308,7 @@ static gboolean remmina_rdp_event_on_draw(GtkWidget* widget, cairo_t* context, R
 		g_free(msg);
 
 	}else  {
-		/* Standard drawing: we copy the surface from RDP */
+		/* Standard drawing: We copy the surface from RDP */
 
 		if (!rfi->surface)
 			return FALSE;
@@ -330,6 +329,7 @@ static gboolean remmina_rdp_event_delayed_monitor_layout(RemminaProtocolWidget*
 {
 	TRACE_CALL(__func__);
 	rfContext* rfi = GET_PLUGIN_DATA(gp);
+
 	RemminaPluginRdpEvent rdp_event = { 0 };
 	GtkAllocation a;
 	gint desktopOrientation, desktopScaleFactor, deviceScaleFactor;
@@ -355,6 +355,13 @@ static gboolean remmina_rdp_event_delayed_monitor_layout(RemminaProtocolWidget*
 		    gpwidth >= 200 && gpwidth < 8192 &&
 		    gpheight >= 200 && gpheight < 8192
 		    ) {
+			if (rfi->rdpgfxchan) {
+				/* Workaround for FreeRDP issue #5417 */
+				if (gpwidth < AVC_MIN_DESKTOP_WIDTH)
+					gpwidth = AVC_MIN_DESKTOP_WIDTH;
+				if (gpheight < AVC_MIN_DESKTOP_HEIGHT)
+					gpheight = AVC_MIN_DESKTOP_HEIGHT;
+			}
 			rdp_event.type = REMMINA_RDP_EVENT_TYPE_SEND_MONITOR_LAYOUT;
 			rdp_event.monitor_layout.width = gpwidth;
 			rdp_event.monitor_layout.height = gpheight;
@@ -661,18 +668,18 @@ static gboolean remmina_rdp_event_on_key(GtkWidget* widget, GdkEventKey* event,
 			}
 		} else {
 			unicode_keyval = gdk_keyval_to_unicode(event->keyval);
-			/* Decide when whe should send a keycode or a unicode character.
-			 * - All non char keys (shift, alt, win) should be sent as keycode
+			/* Decide when whe should send a keycode or a Unicode character.
+			 * - All non char keys (Shift, Alt, Super) should be sent as keycode
 			 * - Space should be sent as keycode (see issue #1364)
-			 * - All special keys (F1-F10, numeric pad, home/end/arrows/pgup/pgdn/ins/del) keycode
-			 * - All key pressed while CTRL or ALT or WIN is down are not decoded by gdk_keyval_to_unicode(), so send it as keycode
+			 * - All special keys (F1-F10, numeric pad, Home/End/Arrows/PgUp/PgDn/Insert/Delete) keycode
+			 * - All key pressed while Ctrl or Alt or Super is down are not decoded by gdk_keyval_to_unicode(), so send it as keycode
 			 * - All keycodes not translatable to unicode chars, as keycode
-			 * - The rest as unicode char
+			 * - The rest as Unicode char
 			 */
-			if (event->keyval >= 0xfe00 ||                                                  // arrows, shift, alt, Fn, num keypad…
-				event->hardware_keycode == 0x41 ||											// space bar
-			    unicode_keyval == 0 ||                                                      // impossible to translate
-			    (event->state & (GDK_MOD1_MASK | GDK_CONTROL_MASK | GDK_SUPER_MASK)) != 0   // a modifier not recognized by gdk_keyval_to_unicode()
+			if (event->keyval >= 0xfe00 ||                                                  // Arrows, Shift, Alt, Fn, num keypad…
+				event->hardware_keycode == 0x41 ||											// Spacebar
+			    unicode_keyval == 0 ||                                                      // Impossible to translate
+			    (event->state & (GDK_MOD1_MASK | GDK_CONTROL_MASK | GDK_SUPER_MASK)) != 0   // A modifier not recognized by gdk_keyval_to_unicode()
 			    ) {
 				scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(event->hardware_keycode);
 				rdp_event.key_event.key_code = scancode & 0xFF;
@@ -795,7 +802,14 @@ void remmina_rdp_event_init(RemminaProtocolWidget* gp)
 	rfi->object_table = g_hash_table_new_full(NULL, NULL, NULL, g_free);
 
 	rfi->display = gdk_display_get_default();
+
+#if GTK_CHECK_VERSION(3, 22, 0)
+	GdkVisual *visual = gdk_screen_get_system_visual(
+			gdk_display_get_default_screen(rfi->display));
+	rfi->bpp = gdk_visual_get_depth (visual);
+#else
 	rfi->bpp = gdk_visual_get_best_depth();
+#endif
 }
 
 
@@ -935,7 +949,7 @@ void remmina_rdp_event_update_scale(RemminaProtocolWidget* gp)
 		/* In non scaled mode, the plugins forces dimensions of drawing area */
 		gtk_widget_set_size_request(rfi->drawing_area, width, height);
 	}
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "update-align");
+	remmina_plugin_service->protocol_plugin_update_align(gp);
 }
 
 static void remmina_rdp_event_connected(RemminaProtocolWidget* gp, RemminaPluginRdpUiObject* ui)
@@ -946,13 +960,14 @@ static void remmina_rdp_event_connected(RemminaProtocolWidget* gp, RemminaPlugin
 
 	gdi = ((rdpContext *)rfi)->gdi;
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "connect");
 	gtk_widget_realize(rfi->drawing_area);
 
 	remmina_rdp_event_create_cairo_surface(rfi);
 	gtk_widget_queue_draw_area(rfi->drawing_area, 0, 0, gdi->width, gdi->height);
 
 	remmina_rdp_event_update_scale(gp);
+    
+    remmina_plugin_service->protocol_plugin_signal_connection_opened(gp);
 }
 
 static void remmina_rdp_event_reconnect_progress(RemminaProtocolWidget* gp, RemminaPluginRdpUiObject* ui)
