@@ -1,5 +1,5 @@
 /*
- * Remmina - The GTK+ Remote Desktop Client
+ * Remmina - The GTK Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
  * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
@@ -41,6 +41,7 @@
 #include <stdlib.h>
 
 #include "config.h"
+#include "remmina_sodium.h"
 #include "remmina.h"
 #include "remmina_exec.h"
 #include "remmina_file_manager.h"
@@ -54,6 +55,7 @@
 #include "remmina_ssh_plugin.h"
 #include "remmina_widget_pool.h"
 #include "remmina/remmina_trace_calls.h"
+#include "rmnews.h"
 #include "remmina_stats_sender.h"
 
 
@@ -78,21 +80,39 @@ gboolean kioskmode;
 
 static GOptionEntry remmina_options[] =
 {
-	{ "about",	  'a', 0,		     G_OPTION_ARG_NONE,		    NULL, N_("Show about dialog"),					       NULL	  },
-	{ "connect",	  'c', 0,		     G_OPTION_ARG_FILENAME,	    NULL, N_("Connect to desktop described by file (.remmina or type supported by plugin)"),				       "FILE"	  },
-	{ G_OPTION_REMAINING, '\0', 0,		     G_OPTION_ARG_FILENAME_ARRAY,   NULL, N_("Connect to desktop described by file (.remmina or type supported by plugin)"),				       "FILE"	  },
-	{ "edit",	  'e', 0,		     G_OPTION_ARG_FILENAME,	    NULL, N_("Edit desktop connection described by file (.remmina or type supported by plugin)"),					       "FILE"	  },
-	{ "help",	  '?', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE,	   	    NULL, NULL,								       NULL	  },
-	{ "kiosk",	  'k', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Start Remmina in Kiosk mode"),				       NULL	  },
-	{ "new",	  'n', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Create a new connection profile"),			       NULL	  },
-	{ "pref",	  'p', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Show preferences dialog page"),				       "PAGENR"	  },
-	{ "plugin",	  'x', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Execute the plugin"),					       "PLUGIN"	  },
-	{ "quit",	  'q', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Quit the application"),					       NULL	  },
-	{ "server",	  's', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Use default server name (for --new)"),			       "SERVER"	  },
-	{ "protocol",	  't', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Use default protocol (for --new)"),			       "PROTOCOL" },
-	{ "icon",	  'i', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Start as tray icon"),					       NULL	  },
-	{ "version",	  'v', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Show the application’s version"),				       NULL	  },
-	{ "full-version", 'V', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Show the application’s version, including the plugin versions"), NULL	  },
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "about",	      'a',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Show \'About\'"),								     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "connect",	      'c',  0,			  G_OPTION_ARG_FILENAME,       NULL, N_("Connect to desktop described in file (.remmina or type supported by plugin)"),	     "FILE"	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ G_OPTION_REMAINING, '\0', 0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Connect to desktop described in file (.remmina or type supported by plugin)"),	     "FILE"	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "edit",	      'e',  0,			  G_OPTION_ARG_FILENAME,       NULL, N_("Edit desktop connection described in file (.remmina or type supported by plugin)"), "FILE"	},
+	{ "help",	      '?',  G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE,	       NULL, NULL,										     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "kiosk",	      'k',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Start in kiosk mode"),							     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "new",	      'n',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Create new connection profile"),						     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "pref",	      'p',  0,			  G_OPTION_ARG_STRING,	       NULL, N_("Show preferences"),						     "PAGENR"	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "plugin",	      'x',  0,			  G_OPTION_ARG_STRING,	       NULL, N_("Run a plugin"),								     "PLUGIN"	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "quit",	      'q',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Quit"),							     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "server",	      's',  0,			  G_OPTION_ARG_STRING,	       NULL, N_("Use default server name (for --new)"),						     "SERVER"	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "protocol",	      't',  0,			  G_OPTION_ARG_STRING,	       NULL, N_("Use default protocol (for --new)"),						     "PROTOCOL" },
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "icon",	      'i',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Start in tray"),								     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "version",	      'v',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Show the application version"),						     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "full-version",     'V',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Show version of the application and its plugins"),		     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "update-profile",   0,    0,			  G_OPTION_ARG_FILENAME,       NULL, N_("Modify connection profile (requires --set-option)"),			     NULL	},
+	// TRANSLATORS: Shown in terminal. Do not use charcters that may be not supported on a terminal
+	{ "set-option",	      0,    0,			  G_OPTION_ARG_STRING_ARRAY,   NULL, N_("Set one or more profile settings, to be used with --update-profile"),		     NULL	},
 	{ NULL }
 };
 
@@ -102,9 +122,9 @@ _gpg_error_to_errno(gcry_error_t e)
 {
 	/* be lazy right now */
 	if (e == GPG_ERR_NO_ERROR)
-		return (0);
+		return 0;
 	else
-		return (EINVAL);
+		return EINVAL;
 }
 #endif /* !WITH_LIBGCRYPT */
 
@@ -120,6 +140,11 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 	gchar *protocol;
 	gchar *server;
 
+#if SODIUM_VERSION_INT >= 90200
+	remmina_sodium_init();
+#endif
+	remmina_pref_init();
+
 	opts = g_application_command_line_get_options_dict(cmdline);
 
 	if (g_variant_dict_lookup_value(opts, "quit", NULL)) {
@@ -165,11 +190,10 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 		if (!g_variant_dict_lookup(opts, "protocol", "&s", &protocol))
 			protocol = NULL;
 
-		if (g_variant_dict_lookup(opts, "server", "&s", &server)) {
+		if (g_variant_dict_lookup(opts, "server", "&s", &server))
 			str = g_strdup_printf("%s,%s", protocol, server);
-		}else  {
+		else
 			str = g_strdup(protocol);
-		}
 
 		remmina_exec_command(REMMINA_COMMAND_NEW, str);
 		g_free(str);
@@ -191,19 +215,8 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 		executed = TRUE;
 	}
 
-	if (g_variant_dict_lookup_value(opts, "version", NULL)) {
-		remmina_exec_command(REMMINA_COMMAND_VERSION, NULL);
-		executed = TRUE;
-	}
-
-	if (g_variant_dict_lookup_value(opts, "full-version", NULL)) {
-		remmina_exec_command(REMMINA_COMMAND_FULL_VERSION, NULL);
-		executed = TRUE;
-	}
-
-	if (!executed) {
+	if (!executed)
 		remmina_exec_command(REMMINA_COMMAND_MAIN, NULL);
-	}
 
 	return status;
 }
@@ -214,9 +227,6 @@ static void remmina_on_startup(GApplication *app)
 
 	RemminaSecretPlugin *secret_plugin;
 
-	remmina_file_manager_init();
-	remmina_pref_init();
-	remmina_plugin_manager_init();
 	remmina_widget_pool_init();
 	remmina_sftp_plugin_register();
 	remmina_ssh_plugin_register();
@@ -226,50 +236,80 @@ static void remmina_on_startup(GApplication *app)
 	gtk_window_set_default_icon_name(REMMINA_APP_ID);
 
 	/* Setting the X11 program class (WM_CLASS) is necessary to group
-	 * windows with .desktop file which has the same StartupWMClass */
+	* windows with .desktop file which has the same StartupWMClass */
 	gdk_set_program_class(REMMINA_APP_ID);
 
 	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(),
-		REMMINA_RUNTIME_DATADIR G_DIR_SEPARATOR_S "icons");
+					  REMMINA_RUNTIME_DATADIR G_DIR_SEPARATOR_S "icons");
 	g_application_hold(app);
 
 	remmina_stats_sender_schedule();
+	rmnews_schedule();
 
-	/* Check for secret plugin and service initialization and show some warnings on the console if
-	 * there is something missing */
+	/* Check for secret plugin and service initialization and show console warnings if
+	 * something is missing */
 	secret_plugin = remmina_plugin_manager_get_secret_plugin();
 	if (!secret_plugin) {
-		g_print("WARNING: Remmina is running without a secret plugin. Passwords will be saved in a less secure way.\n");
+		g_print("Warning: Remmina is running without a secret plugin. Passwords will be saved in a less secure way.\n");
 	} else {
-		if (!secret_plugin->is_service_available()) {
-			g_print("WARNING: Remmina is running with a secret plugin, but it cannot connect to a secret service.\n");
-		}
+		if (!secret_plugin->is_service_available())
+			g_print("Warning: Remmina is running with a secrecy plugin, but it cannot connect to a secrecy service.\n");
 	}
 
+	remmina_exec_command(REMMINA_COMMAND_AUTOSTART, NULL);
 }
 
-static gint remmina_on_local_cmdline(GApplication *app, GVariantDict *options, gpointer user_data)
+static gint remmina_on_local_cmdline(GApplication *app, GVariantDict *opts, gpointer user_data)
 {
 	TRACE_CALL(__func__);
 
 	int status = -1;
+	gchar *str;
+	gchar **settings;
 
 	/* Here you handle any command line options that you want to be executed
-	 * from command line, one time, and than exit */
+	 * in the local instance (the non-unique instance) */
 
+	if (g_variant_dict_lookup_value(opts, "version", NULL)) {
+		remmina_exec_command(REMMINA_COMMAND_VERSION, NULL);
+		status = 0;
+	}
+
+	if (g_variant_dict_lookup_value(opts, "full-version", NULL)) {
+		remmina_exec_command(REMMINA_COMMAND_FULL_VERSION, NULL);
+		status = 0;
+	}
+
+	if (g_variant_dict_lookup(opts, "update-profile", "^&ay", &str)) { /* ^&ay no need to free */
+		if (g_variant_dict_lookup(opts, "set-option", "^a&s", &settings)) {
+			if (settings != NULL) {
+				status = remmina_exec_set_setting(str, settings);
+				g_free(settings);
+			} else {
+				status = 1;
+			}
+		} else {
+			status = 1;
+			g_print("Error: --update-profile requires --set-option\n");
+		}
+	}
+
+	/* Returning a non negative value here makes the application exit */
 	return status;
 }
 
-int main(int argc, char* argv[])
+int main(int argc, char *argv[])
 {
 	TRACE_CALL(__func__);
 	GtkApplication *app;
 	const gchar *app_id;
 	int status;
 
+	g_unsetenv ("GDK_CORE_DEVICE_EVENTS");
+
 	/* Enable wayland backend only after GTK 3.22.27 or the clipboard
 	 * will not work. See GTK bug 790031 */
-	if (remmina_gtk_check_version(3,22,27))
+	if (remmina_gtk_check_version(3, 22, 27))
 		gdk_set_allowed_backends("wayland,x11,broadway,quartz,mir");
 	else
 		gdk_set_allowed_backends("x11,broadway,quartz,mir");
@@ -284,9 +324,8 @@ int main(int argc, char* argv[])
 # if GCRYPT_VERSION_NUMBER < 0x010600
 	gcry_error_t e;
 	if (!gcrypt_thread_initialized) {
-		if ((e = gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread)) != GPG_ERR_NO_ERROR) {
-			return (-1);
-		}
+		if ((e = gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread)) != GPG_ERR_NO_ERROR)
+			return -1;
 		gcrypt_thread_initialized++;
 	}
 #endif  /* !GCRYPT_VERSION_NUMBER */
@@ -295,9 +334,15 @@ int main(int argc, char* argv[])
 	gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
 #endif  /* !HAVE_LIBGCRYPT */
 
+	/* Initialize some Remmina parts needed also on a local instance for correct handle-local-options */
+	remmina_pref_init();
+	remmina_file_manager_init();
+	remmina_plugin_manager_init();
+
+
 	app_id = g_application_id_is_valid(REMMINA_APP_ID) ? REMMINA_APP_ID : NULL;
 	app = gtk_application_new(app_id, G_APPLICATION_HANDLES_COMMAND_LINE);
-#if !GTK_CHECK_VERSION(4, 0, 0) /* This is not needed anymore starting from Gtk+ 4 */
+#if !GTK_CHECK_VERSION(4, 0, 0) /* This is not needed anymore starting from GTK 4 */
 	g_set_prgname(app_id);
 #endif
 	g_signal_connect(app, "startup", G_CALLBACK(remmina_on_startup), NULL);
