@@ -60,6 +60,7 @@
 #include "remmina_pref.h"
 #include "remmina_main.h"
 #include "remmina_masterthread_exec.h"
+#include "remmina_utils.h"
 #include "remmina/remmina_trace_calls.h"
 
 #define MIN_WINDOW_WIDTH 10
@@ -67,7 +68,7 @@
 
 static struct timespec times[2];
 
-static RemminaFile*
+static RemminaFile *
 remmina_file_new_empty(void)
 {
 	TRACE_CALL(__func__);
@@ -83,7 +84,7 @@ remmina_file_new_empty(void)
 	return remminafile;
 }
 
-RemminaFile*
+RemminaFile *
 remmina_file_new(void)
 {
 	TRACE_CALL(__func__);
@@ -95,7 +96,7 @@ remmina_file_new(void)
 	if (remminafile) {
 		g_free(remminafile->filename);
 		remminafile->filename = NULL;
-	}else {
+	} else {
 		remminafile = remmina_file_new_empty();
 	}
 
@@ -105,16 +106,66 @@ remmina_file_new(void)
 void remmina_file_generate_filename(RemminaFile *remminafile)
 {
 	TRACE_CALL(__func__);
-	GTimeVal gtime;
 	GDir *dir;
 
+	/* File name restrictions:
+	 * - Do not start with space
+	 * - Do not end with space or dot
+	 * - No more than 255 chars
+	 * - Do not contain \0
+	 * - Avoid % and $
+	 * - Avoid underscores and spaces for interoperabiility with everything else
+	 * - Better all lowercase
+	 */
+	gchar *invalid_chars = "\\%|/$?<>:*. \"";
+	GString *filenamestr;
+	gchar *filename;
+	const gchar *s;
+
+
+	/** functions we can use
+	 * g_strstrip( string )
+	 *	Removes leading and trailing whitespace from a string
+	 * g_strdelimit (str, invalid_chars, '-'))
+	 *	Convert each invalid_chars in a hyphen
+	 * g_ascii_strdown(string)
+	 *	all lowercase
+	 * To be safe we should remove control characters as well (but I'm lazy)
+	 * https://rosettacode.org/wiki/Strip_control_codes_and_extended_characters_from_a_string#C
+	 * g_utf8_strncpy (gchar *dest, const gchar *src, gsize n);
+	 *	copies a given number of characters instead of a given number of bytes. The src string must be valid UTF-8 encoded text.
+	 * g_utf8_validate (const gchar *str, gssize max_len, const gchar **end);
+	 *	Validates UTF-8 encoded text.
+	 */
+
 	g_free(remminafile->filename);
-	g_get_current_time(&gtime);
+
+	filenamestr = g_string_new(g_strdup_printf("%s",
+						   remmina_pref.remmina_file_name));
+	if ((s = remmina_file_get_string(remminafile, "name")) == NULL) s = "name";
+	if (g_strstr_len(filenamestr->str, -1, "%N") != NULL)
+		remmina_utils_string_replace_all(filenamestr, "%N", s);
+
+	if ((s = remmina_file_get_string(remminafile, "group")) == NULL) s = "group";
+	if (g_strstr_len(filenamestr->str, -1, "%G") != NULL)
+		remmina_utils_string_replace_all(filenamestr, "%G", s);
+
+	if ((s = remmina_file_get_string(remminafile, "protocol")) == NULL) s = "proto";
+	if (g_strstr_len(filenamestr->str, -1, "%P") != NULL)
+		remmina_utils_string_replace_all(filenamestr, "%P", s);
+
+	if ((s = remmina_file_get_string(remminafile, "server")) == NULL) s = "host";
+	if (g_strstr_len(filenamestr->str, -1, "%h") != NULL)
+		remmina_utils_string_replace_all(filenamestr, "%h", s);
+
+	s = NULL;
+
+	filename = g_strdelimit(g_ascii_strdown(g_strstrip(g_string_free(filenamestr, FALSE)), -1),
+				invalid_chars, '-');
 
 	dir = g_dir_open(remmina_file_get_datadir(), 0, NULL);
 	if (dir != NULL)
-		remminafile->filename = g_strdup_printf("%s/%li%03li.remmina", remmina_file_get_datadir(), gtime.tv_sec,
-			gtime.tv_usec / 1000);
+		remminafile->filename = g_strdup_printf("%s/%s.remmina", remmina_file_get_datadir(), filename);
 	else
 		remminafile->filename = NULL;
 	g_dir_close(dir);
@@ -127,31 +178,36 @@ void remmina_file_set_filename(RemminaFile *remminafile, const gchar *filename)
 	remminafile->filename = g_strdup(filename);
 }
 
-const gchar*
+const gchar *
 remmina_file_get_filename(RemminaFile *remminafile)
 {
 	TRACE_CALL(__func__);
 	return remminafile->filename;
 }
 
-RemminaFile*
+RemminaFile *
 remmina_file_copy(const gchar *filename)
 {
 	TRACE_CALL(__func__);
 	RemminaFile *remminafile;
 
 	remminafile = remmina_file_load(filename);
-	if (remminafile) {
+	remmina_file_set_string(remminafile,
+			"name",
+			g_strdup_printf(
+				"COPY %s",
+				remmina_file_get_string(remminafile, "name")));
+
+	if (remminafile)
 		remmina_file_generate_filename(remminafile);
-	}
 
 	return remminafile;
 }
 
-static const RemminaProtocolSetting* find_protocol_setting(const gchar *name, RemminaProtocolPlugin* protocol_plugin)
+const RemminaProtocolSetting *find_protocol_setting(const gchar *name, RemminaProtocolPlugin *protocol_plugin)
 {
 	TRACE_CALL(__func__);
-	const RemminaProtocolSetting* setting_iter;
+	const RemminaProtocolSetting *setting_iter;
 
 	if (protocol_plugin == NULL)
 		return NULL;
@@ -175,10 +231,9 @@ static const RemminaProtocolSetting* find_protocol_setting(const gchar *name, Re
 	}
 
 	return NULL;
-
 }
 
-RemminaFile*
+RemminaFile *
 remmina_file_load(const gchar *filename)
 {
 	TRACE_CALL(__func__);
@@ -190,17 +245,19 @@ remmina_file_load(const gchar *filename)
 	gchar *resolution_str;
 	gint i;
 	gchar *s, *sec;
-	RemminaProtocolPlugin* protocol_plugin;
+	RemminaProtocolPlugin *protocol_plugin;
 	RemminaSecretPlugin *secret_plugin;
 	gboolean secret_service_available;
 	int w, h;
 
 	gkeyfile = g_key_file_new();
 
-	if (!g_key_file_load_from_file(gkeyfile, filename, G_KEY_FILE_NONE, NULL)) {
-		g_key_file_free(gkeyfile);
-		g_printf("WARNING: unable to load remmina profile file %s: g_key_file_load_from_file() returned NULL.\n", filename);
-		return NULL;
+	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS)) {
+		if (!g_key_file_load_from_file(gkeyfile, filename, G_KEY_FILE_NONE, NULL)) {
+			g_key_file_free(gkeyfile);
+			g_debug("Unable to load remmina profile file %s: g_key_file_load_from_file() returned NULL.\n", filename);
+			return NULL;
+		}
 	}
 
 	if (g_key_file_has_key(gkeyfile, "remmina", "name", NULL)) {
@@ -211,7 +268,7 @@ remmina_file_load(const gchar *filename)
 		/* Identify the protocol plugin and get pointers to its RemminaProtocolSetting structs */
 		proto = g_key_file_get_string(gkeyfile, "remmina", "protocol", NULL);
 		if (proto) {
-			protocol_plugin = (RemminaProtocolPlugin*)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
+			protocol_plugin = (RemminaProtocolPlugin *)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
 			g_free(proto);
 		}
 
@@ -232,14 +289,14 @@ remmina_file_load(const gchar *filename)
 							/* Annotate in spsettings that this value comes from secret_plugin */
 							g_hash_table_insert(remminafile->spsettings, g_strdup(key), NULL);
 							g_free(sec);
-						}else {
+						} else {
 							remmina_file_set_string(remminafile, key, s);
 						}
-					}else {
+					} else {
 						remmina_file_set_string_ref(remminafile, key, remmina_crypt_decrypt(s));
 					}
 					g_free(s);
-				}else {
+				} else {
 					/* If we find "resolution", then we split it in two */
 					if (strcmp(key, "resolution") == 0) {
 						resolution_str = g_key_file_get_string(gkeyfile, "remmina", key, NULL);
@@ -251,16 +308,16 @@ remmina_file_load(const gchar *filename)
 							remmina_file_set_string_ref(remminafile, "resolution_height", NULL);
 						}
 						g_free(resolution_str);
-					}else {
+					} else {
 						remmina_file_set_string_ref(remminafile, key,
-							g_key_file_get_string(gkeyfile, "remmina", key, NULL));
+									    g_key_file_get_string(gkeyfile, "remmina", key, NULL));
 					}
 				}
 			}
 			g_strfreev(keys);
 		}
-	}else {
-		g_printf("WARNING: unable to load remmina profile file %s: cannot find key name= in section remmina.\n", filename);
+	} else {
+		g_debug("Unable to load remmina profile file %s: cannot find key name= in section remmina.\n", filename);
 		remminafile = NULL;
 	}
 
@@ -278,7 +335,7 @@ void remmina_file_set_string(RemminaFile *remminafile, const gchar *setting, con
 void remmina_file_set_string_ref(RemminaFile *remminafile, const gchar *setting, gchar *value)
 {
 	TRACE_CALL(__func__);
-	const gchar* message;
+	const gchar *message;
 
 	if (value) {
 		/* We refuse to accept to set the "resolution" field */
@@ -289,26 +346,26 @@ void remmina_file_set_string_ref(RemminaFile *remminafile, const gchar *setting,
 			return;
 		}
 		g_hash_table_insert(remminafile->settings, g_strdup(setting), value);
-	}else {
+	} else {
 		g_hash_table_insert(remminafile->settings, g_strdup(setting), g_strdup(""));
 	}
 }
 
-const gchar*
+const gchar *
 remmina_file_get_string(RemminaFile *remminafile, const gchar *setting)
 {
 	TRACE_CALL(__func__);
 	gchar *value;
-	const gchar* message;
+	const gchar *message;
 
 	/* Returned value is a pointer to the string stored on the hash table,
 	 * please do not free it or the hash table will contain invalid pointer */
-	if ( !remmina_masterthread_exec_is_main_thread() ) {
+	if (!remmina_masterthread_exec_is_main_thread()) {
 		/* Allow the execution of this function from a non main thread
 		 * (plugins needs it to have user credentials)*/
 		RemminaMTExecData *d;
 		const gchar *retval;
-		d = (RemminaMTExecData*)g_malloc( sizeof(RemminaMTExecData) );
+		d = (RemminaMTExecData *)g_malloc(sizeof(RemminaMTExecData));
 		d->func = FUNC_FILE_GET_STRING;
 		d->p.file_get_string.remminafile = remminafile;
 		d->p.file_get_string.setting = setting;
@@ -325,18 +382,18 @@ remmina_file_get_string(RemminaFile *remminafile, const gchar *setting)
 		return NULL;
 	}
 
-	value = (gchar*)g_hash_table_lookup(remminafile->settings, setting);
+	value = (gchar *)g_hash_table_lookup(remminafile->settings, setting);
 	return value && value[0] ? value : NULL;
 }
 
-gchar*
+gchar *
 remmina_file_get_secret(RemminaFile *remminafile, const gchar *setting)
 {
 	TRACE_CALL(__func__);
 
 	/* This function is in the RemminaPluginService table, we cannot remove it
 	 * without breaking plugin API */
-	printf("WARNING: remmina_file_get_secret(remminafile,\"%s\") is deprecated and must not be called. Use remmina_file_get_string() and do not deallocate returned memory.\n", setting);
+	g_warning("remmina_file_get_secret(remminafile,\"%s\") is deprecated and must not be called. Use remmina_file_get_string() and do not deallocate returned memory.\n", setting);
 	return g_strdup(remmina_file_get_string(remminafile, setting));
 }
 
@@ -355,7 +412,7 @@ gint remmina_file_get_int(RemminaFile *remminafile, const gchar *setting, gint d
 	return value == NULL ? default_value : (value[0] == 't' ? TRUE : atoi(value));
 }
 
-static GKeyFile*
+static GKeyFile *
 remmina_file_get_keyfile(RemminaFile *remminafile)
 {
 	TRACE_CALL(__func__);
@@ -388,12 +445,14 @@ void remmina_file_save(RemminaFile *remminafile)
 	TRACE_CALL(__func__);
 	RemminaSecretPlugin *secret_plugin;
 	gboolean secret_service_available;
-	RemminaProtocolPlugin* protocol_plugin;
+	RemminaProtocolPlugin *protocol_plugin;
 	GHashTableIter iter;
 	const gchar *key, *value;
 	gchar *s, *proto, *content;
+	gint nopasswdsave;
 	GKeyFile *gkeyfile;
 	gsize length = 0;
+	GError *err = NULL;
 
 	if (remminafile->prevent_saving)
 		return;
@@ -401,12 +460,15 @@ void remmina_file_save(RemminaFile *remminafile)
 	if ((gkeyfile = remmina_file_get_keyfile(remminafile)) == NULL)
 		return;
 
+	g_debug("Saving profile");
+	/* get disablepasswordstoring */
+	nopasswdsave = remmina_file_get_int(remminafile, "disablepasswordstoring", 0);
 	/* Identify the protocol plugin and get pointers to its RemminaProtocolSetting structs */
-	proto = (gchar*)g_hash_table_lookup(remminafile->settings, "protocol");
+	proto = (gchar *)g_hash_table_lookup(remminafile->settings, "protocol");
 	if (proto) {
-		protocol_plugin = (RemminaProtocolPlugin*)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
+		protocol_plugin = (RemminaProtocolPlugin *)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
 	} else {
-		printf("Remmina WARNING: saving settings for unknown protocol, because remminafile has non proto key\n");
+		g_warning("Saving settings for unknown protocol, because remminafile has non proto key\n");
 		protocol_plugin = NULL;
 	}
 
@@ -414,30 +476,40 @@ void remmina_file_save(RemminaFile *remminafile)
 	secret_service_available = secret_plugin && secret_plugin->is_service_available();
 
 	g_hash_table_iter_init(&iter, remminafile->settings);
-	while (g_hash_table_iter_next(&iter, (gpointer*)&key, (gpointer*)&value)) {
+	while (g_hash_table_iter_next(&iter, (gpointer *)&key, (gpointer *)&value)) {
 		if (remmina_plugin_manager_is_encrypted_setting(protocol_plugin, key)) {
 			if (remminafile->filename && g_strcmp0(remminafile->filename, remmina_pref_file)) {
-				if (secret_service_available) {
+				if (secret_service_available && nopasswdsave == 0) {
+					g_debug ("We have a secret and disablepasswordstoring=0");
 					if (value && value[0]) {
-						if (g_strcmp0(value, ".") != 0) {
+						if (g_strcmp0(value, ".") != 0)
 							secret_plugin->store_password(remminafile, key, value);
-						}
 						g_key_file_set_string(gkeyfile, "remmina", key, ".");
-					}else {
+					} else {
 						g_key_file_set_string(gkeyfile, "remmina", key, "");
 						secret_plugin->delete_password(remminafile, key);
 					}
-				}else {
-					if (value && value[0]) {
+				} else {
+					g_debug ("We have a password and disablepasswordstoring=0");
+					if (value && value[0] && nopasswdsave == 0) {
 						s = remmina_crypt_encrypt(value);
 						g_key_file_set_string(gkeyfile, "remmina", key, s);
 						g_free(s);
-					}else {
+					} else {
 						g_key_file_set_string(gkeyfile, "remmina", key, "");
 					}
 				}
+				if (secret_service_available && nopasswdsave == 1) {
+					if (value && value[0]) {
+						if (g_strcmp0(value, ".") != 0) {
+							g_debug ("Deleting the secret in the keyring as disablepasswordstoring=1");
+							secret_plugin->delete_password(remminafile, key);
+							g_key_file_set_string(gkeyfile, "remmina", key, ".");
+						}
+					}
+				}
 			}
-		}else {
+		} else {
 			g_key_file_set_string(gkeyfile, "remmina", key, value);
 		}
 	}
@@ -447,7 +519,13 @@ void remmina_file_save(RemminaFile *remminafile)
 
 	/* Store gkeyfile to disk (password are already sent to keyring) */
 	content = g_key_file_to_data(gkeyfile, &length, NULL);
-	g_file_set_contents(remminafile->filename, content, length, NULL);
+	if (g_file_set_contents(remminafile->filename, content, length, &err)) {
+		g_debug("Profile saved");
+	} else {
+		g_warning("Remmina connection profile cannot be saved, with error %d (%s)", err->code, err->message);
+	}
+	if (err != NULL)
+		g_error_free (err);
 
 	g_free(content);
 	g_key_file_free(gkeyfile);
@@ -455,14 +533,14 @@ void remmina_file_save(RemminaFile *remminafile)
 	remmina_main_update_file_datetime(remminafile);
 }
 
-void remmina_file_store_secret_plugin_password(RemminaFile *remminafile, const gchar* key, const gchar* value)
+void remmina_file_store_secret_plugin_password(RemminaFile *remminafile, const gchar *key, const gchar *value)
 {
 	TRACE_CALL(__func__);
 
 	/* Only change the password in the keyring. This function
 	 * is a shortcut which avoids updating of date/time of .pref file
 	 * when possible, and is used by the mpchanger */
-	RemminaSecretPlugin* plugin;
+	RemminaSecretPlugin *plugin;
 
 	if (g_hash_table_lookup_extended(remminafile->spsettings, g_strdup(key), NULL, NULL)) {
 		plugin = remmina_plugin_manager_get_secret_plugin();
@@ -473,7 +551,7 @@ void remmina_file_store_secret_plugin_password(RemminaFile *remminafile, const g
 	}
 }
 
-RemminaFile*
+RemminaFile *
 remmina_file_dup(RemminaFile *remminafile)
 {
 	TRACE_CALL(__func__);
@@ -485,28 +563,27 @@ remmina_file_dup(RemminaFile *remminafile)
 	dupfile->filename = g_strdup(remminafile->filename);
 
 	g_hash_table_iter_init(&iter, remminafile->settings);
-	while (g_hash_table_iter_next(&iter, (gpointer*)&key, (gpointer*)&value)) {
+	while (g_hash_table_iter_next(&iter, (gpointer *)&key, (gpointer *)&value))
 		remmina_file_set_string(dupfile, key, value);
-	}
 
 	return dupfile;
 }
 
-const gchar*
+const gchar *
 remmina_file_get_icon_name(RemminaFile *remminafile)
 {
 	TRACE_CALL(__func__);
 	RemminaProtocolPlugin *plugin;
 
-	plugin = (RemminaProtocolPlugin*)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL,
-		remmina_file_get_string(remminafile, "protocol"));
+	plugin = (RemminaProtocolPlugin *)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL,
+									    remmina_file_get_string(remminafile, "protocol"));
 	if (!plugin)
-		return REMMINA_APP_ID;
+		return g_strconcat (REMMINA_APP_ID, "-symbolic", NULL);
 
-	return (remmina_file_get_int(remminafile, "ssh_enabled", FALSE) ? plugin->icon_name_ssh : plugin->icon_name);
+	return remmina_file_get_int(remminafile, "ssh_enabled", FALSE) ? plugin->icon_name_ssh : plugin->icon_name;
 }
 
-RemminaFile*
+RemminaFile *
 remmina_file_dup_temp_protocol(RemminaFile *remminafile, const gchar *new_protocol)
 {
 	TRACE_CALL(__func__);
@@ -526,38 +603,37 @@ void remmina_file_delete(const gchar *filename)
 
 	remminafile = remmina_file_load(filename);
 	if (remminafile) {
-		remmina_file_unsave_password(remminafile);
+		remmina_file_unsave_passwords(remminafile);
 		remmina_file_free(remminafile);
 	}
 	g_unlink(filename);
 }
 
-void remmina_file_unsave_password(RemminaFile *remminafile)
+void remmina_file_unsave_passwords(RemminaFile *remminafile)
 {
 	/* Delete all saved secrets for this profile */
 
 	TRACE_CALL(__func__);
-	const RemminaProtocolSetting* setting_iter;
-	RemminaProtocolPlugin* protocol_plugin;
+	const RemminaProtocolSetting *setting_iter;
+	RemminaProtocolPlugin *protocol_plugin;
 	gchar *proto;
 	protocol_plugin = NULL;
 
 	remmina_file_set_string(remminafile, "password", NULL);
 
-	proto = (gchar*)g_hash_table_lookup(remminafile->settings, "protocol");
+	proto = (gchar *)g_hash_table_lookup(remminafile->settings, "protocol");
 	if (proto) {
-		protocol_plugin = (RemminaProtocolPlugin*)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
+		protocol_plugin = (RemminaProtocolPlugin *)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
 		if (protocol_plugin) {
 			setting_iter = protocol_plugin->basic_settings;
 			if (setting_iter) {
 				while (setting_iter->type != REMMINA_PROTOCOL_SETTING_TYPE_END) {
 					g_debug("setting name: %s", setting_iter->name);
 					if (setting_iter->name == NULL) {
-						g_warning("Internal error: a setting name in protocol plugin %s is null. Please fix RemminaProtocolSetting struct content.", proto);
-					}else {
-						if (remmina_plugin_manager_is_encrypted_setting(protocol_plugin, setting_iter->name)) {
+						g_error("Internal error: a setting name in protocol plugin %s is null. Please fix RemminaProtocolSetting struct content.", proto);
+					} else {
+						if (remmina_plugin_manager_is_encrypted_setting(protocol_plugin, setting_iter->name))
 							remmina_file_set_string(remminafile, remmina_plugin_manager_get_canonical_setting_name(setting_iter), NULL);
-						}
 					}
 					setting_iter++;
 				}
@@ -565,9 +641,8 @@ void remmina_file_unsave_password(RemminaFile *remminafile)
 			setting_iter = protocol_plugin->advanced_settings;
 			if (setting_iter) {
 				while (setting_iter->type != REMMINA_PROTOCOL_SETTING_TYPE_END) {
-					if (remmina_plugin_manager_is_encrypted_setting(protocol_plugin, setting_iter->name)) {
+					if (remmina_plugin_manager_is_encrypted_setting(protocol_plugin, setting_iter->name))
 						remmina_file_set_string(remminafile, remmina_plugin_manager_get_canonical_setting_name(setting_iter), NULL);
-					}
 					setting_iter++;
 				}
 			}
@@ -586,7 +661,7 @@ void remmina_file_unsave_password(RemminaFile *remminafile)
  * @return A date string in the form "%d/%m/%Y %H:%M:%S".
  * @todo This should be moved to remmina_utils.c
  */
-gchar*
+gchar *
 remmina_file_get_datetime(RemminaFile *remminafile)
 {
 	TRACE_CALL(__func__);
@@ -595,7 +670,7 @@ remmina_file_get_datetime(RemminaFile *remminafile)
 	GFileInfo *info;
 
 	struct timeval tv;
-	struct tm* ptm;
+	struct tm *ptm;
 	char time_string[256];
 
 	guint64 mtime;
@@ -604,10 +679,10 @@ remmina_file_get_datetime(RemminaFile *remminafile)
 	file = g_file_new_for_path(remminafile->filename);
 
 	info = g_file_query_info(file,
-		G_FILE_ATTRIBUTE_TIME_MODIFIED,
-		G_FILE_QUERY_INFO_NONE,
-		NULL,
-		NULL);
+				 G_FILE_ATTRIBUTE_TIME_MODIFIED,
+				 G_FILE_QUERY_INFO_NONE,
+				 NULL,
+				 NULL);
 
 	g_object_unref(file);
 
@@ -661,4 +736,3 @@ remmina_file_touch(RemminaFile *remminafile)
 
 	remmina_file_touch(remminafile);
 }
-
