@@ -434,11 +434,11 @@ void remmina_icon_init(void)
 	gboolean sni_supported;
 	char msg[200];
 
-	if (remmina_pref.dark_tray_icon) {
+	if (remmina_pref.dark_tray_icon)
 		g_stpcpy(remmina_panel, "remmina-panel-inverted");
-	}else  {
+	else
 		g_stpcpy(remmina_panel, "remmina-panel");
-	}
+
 
 	/* Print on stdout the availability of appindicators on DBUS */
 	sni_supported = remmina_sysinfo_is_appindicator_available();
