@@ -35,9 +35,11 @@
  */
 
 #include "config.h"
-#include <gtk/gtk.h>
+#include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "remmina.h"
 #include "remmina_string_array.h"
@@ -57,12 +59,12 @@
 #include "remmina_exec.h"
 #include "remmina_mpchange.h"
 #include "remmina_external_tools.h"
+#include "remmina_unlock.h"
 #include "remmina/remmina_trace_calls.h"
-#include "remmina_stats_sender.h"
 
 static RemminaMain *remminamain;
 
-#define GET_OBJECT(object_name) gtk_builder_get_object(remminamain->builder, object_name)
+#define RM_GET_OBJECT(object_name) gtk_builder_get_object(remminamain->builder, object_name)
 
 enum {
 	PROTOCOL_COLUMN,
@@ -75,8 +77,20 @@ enum {
 	N_COLUMNS
 };
 
+static
+const gchar *supported_mime_types[] = {
+  "x-scheme-handler/rdp",
+  "x-scheme-handler/spice",
+  "x-scheme-handler/vnc",
+  "x-scheme-handler/remmina",
+  "application/x-remmina",
+  NULL
+};
+
 static GActionEntry main_actions[] = {
 	{   "about",       remmina_main_on_action_application_about,         NULL, NULL, NULL },
+	{   "news",        remmina_main_on_action_application_news,          NULL, NULL, NULL },
+	{   "default",     remmina_main_on_action_application_default,       NULL, NULL, NULL },
 	{   "mpchange",    remmina_main_on_action_application_mpchange,      NULL, NULL, NULL },
 	{   "plugins",     remmina_main_on_action_application_plugins,       NULL, NULL, NULL },
 	{   "preferences", remmina_main_on_action_application_preferences,   NULL, NULL, NULL },
@@ -94,6 +108,8 @@ static GActionEntry main_actions[] = {
 	{   "wiki",        remmina_main_on_action_help_wiki,                 NULL, NULL, NULL },
 	{   "export",      remmina_main_on_action_tools_export,              NULL, NULL, NULL },
 	{   "import",      remmina_main_on_action_tools_import,              NULL, NULL, NULL },
+	{   "expand",      remmina_main_on_action_expand,                    NULL, NULL, NULL },
+	{   "collapse",    remmina_main_on_action_collapse,                    NULL, NULL, NULL },
 };
 
 static GtkTargetEntry remmina_drop_types[] =
@@ -182,6 +198,27 @@ gboolean remmina_main_on_delete_event(GtkWidget *widget, GdkEvent *event, gpoint
 	return FALSE;
 }
 
+/**
+ * Called after the main window is destroyed, will cleanup remminamain object and related data
+ */
+static gboolean remmina_main_rmodestroy(gpointer data)
+{
+	g_free(remmina_pref.expanded_group);
+	remmina_pref.expanded_group = remmina_string_array_to_string(remminamain->priv->expanded_group);
+	remmina_string_array_free(remminamain->priv->expanded_group);
+	remminamain->priv->expanded_group = NULL;
+	if (remminamain->priv->file_model)
+		g_object_unref(G_OBJECT(remminamain->priv->file_model));
+
+	g_object_unref(G_OBJECT(remminamain->priv->file_model_filter));
+	g_object_unref(remminamain->builder);
+	g_free(remminamain->priv->selected_filename);
+	g_free(remminamain->priv->selected_name);
+	g_free(remminamain->priv);
+	g_free(remminamain);
+	remminamain = NULL;
+	return FALSE;
+}
 /**
  * Called when the main window is destroyed via a call from gtk_widget_destroy()
  */
@@ -190,23 +227,11 @@ void remmina_main_destroy()
 	TRACE_CALL(__func__);
 
 	if (remminamain) {
+		/* Schedule remminamain object destruction after
+		 * GTK will finish to destroy the main window */
 		if (remminamain->window)
 			remmina_main_save_before_destroy();
-		g_free(remmina_pref.expanded_group);
-		remmina_pref.expanded_group = remmina_string_array_to_string(remminamain->priv->expanded_group);
-		remmina_string_array_free(remminamain->priv->expanded_group);
-		remminamain->priv->expanded_group = NULL;
-
-		if (remminamain->priv->file_model)
-			g_object_unref(G_OBJECT(remminamain->priv->file_model));
-		g_object_unref(G_OBJECT(remminamain->priv->file_model_filter));
-		g_object_unref(remminamain->builder);
-		g_free(remminamain->priv->selected_filename);
-		g_free(remminamain->priv->selected_name);
-		g_free(remminamain->priv);
-		g_free(remminamain);
-
-		remminamain = NULL;
+		g_idle_add(remmina_main_rmodestroy, NULL);
 	}
 }
 
@@ -219,6 +244,23 @@ static void remmina_main_clear_selection_data(void)
 	remminamain->priv->selected_name = NULL;
 }
 
+static void remmina_main_check_env()
+{
+	GtkBuilder *dlgbuilder = NULL;
+	GtkWidget *dlg;
+	GtkWindow *parent;
+	GtkWidget* dsa;
+	int result;
+	gint64 nowsec;
+	static gboolean shown_once = FALSE;
+	#define SUPPRESS_DAYS 20
+
+	if (shown_once)
+		return;
+	else
+		shown_once = TRUE;
+}
+
 #ifdef SNAP_BUILD
 
 static void remmina_main_show_snap_welcome()
@@ -657,6 +699,9 @@ void remmina_main_on_action_connection_external_tools(GSimpleAction *action, GVa
 	if (!remminamain->priv->selected_filename)
 		return;
 
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	remmina_external_tools_from_filename(remminamain, remminamain->priv->selected_filename);
 }
 
@@ -687,6 +732,9 @@ void remmina_main_on_action_application_mpchange(GSimpleAction *action, GVariant
 		}
 	}
 
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	remmina_mpchange_schedule(TRUE, group, domain, username, "");
 
 	if (remminafile != NULL)
@@ -699,6 +747,9 @@ void remmina_main_on_action_connection_new(GSimpleAction *action, GVariant *para
 	TRACE_CALL(__func__);
 	GtkWidget *widget;
 
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	widget = remmina_file_editor_new();
 	g_signal_connect(G_OBJECT(widget), "destroy", G_CALLBACK(remmina_main_file_editor_destroy), remminamain);
 	gtk_window_set_transient_for(GTK_WINDOW(widget), remminamain->window);
@@ -739,6 +790,9 @@ void remmina_main_on_action_connection_copy(GSimpleAction *action, GVariant *par
 	TRACE_CALL(__func__);
 	GtkWidget *widget;
 
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	if (!remminamain->priv->selected_filename)
 		return;
 
@@ -748,7 +802,7 @@ void remmina_main_on_action_connection_copy(GSimpleAction *action, GVariant *par
 		gtk_window_set_transient_for(GTK_WINDOW(widget), remminamain->window);
 		gtk_widget_show(widget);
 	}
-/* Select the file previously selected */
+	/* Select the file previously selected */
 	if (remminamain->priv->selected_filename) {
 		remmina_main_select_file(remminamain->priv->selected_filename);
 	}
@@ -759,6 +813,9 @@ void remmina_main_on_action_connection_edit(GSimpleAction *action, GVariant *par
 	TRACE_CALL(__func__);
 	GtkWidget *widget;
 
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	if (!remminamain->priv->selected_filename)
 		return;
 
@@ -782,8 +839,11 @@ void remmina_main_on_action_connection_delete(GSimpleAction *action, GVariant *p
 	if (!remminamain->priv->selected_filename)
 		return;
 
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-		_("Are you sure to delete '%s'"), remminamain->priv->selected_name);
+		_("Are you sure you want to delete \"%s\"?"), remminamain->priv->selected_name);
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES) {
 		delfilename = g_strdup(remminamain->priv->selected_filename);
 		remmina_file_delete(delfilename);
@@ -798,16 +858,44 @@ void remmina_main_on_action_connection_delete(GSimpleAction *action, GVariant *p
 void remmina_main_on_action_application_preferences(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	TRACE_CALL(__func__);
+	if (remmina_unlock_new(remminamain->window) == 0)
+		return;
 	GtkDialog *dialog = remmina_pref_dialog_new(0, remminamain->window);
 	gtk_dialog_run(dialog);
 	gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
+void remmina_main_on_action_application_default(GSimpleAction *action, GVariant *param, gpointer data)
+{
+	TRACE_CALL(__func__);
+	g_autoptr(GError) error = NULL;
+	GDesktopAppInfo *desktop_info;
+	GAppInfo *info = NULL;
+	g_autofree gchar *id = g_strconcat (REMMINA_APP_ID, ".desktop", NULL);
+	int i;
+
+	desktop_info = g_desktop_app_info_new (id);
+	if (!desktop_info)
+		return;
+
+	info = G_APP_INFO (desktop_info);
+
+	for (i = 0; supported_mime_types[i]; i++) {
+		if (!g_app_info_set_as_default_for_type (info, supported_mime_types[i], &error))
+			g_warning ("Failed to set '%s' as the default application for secondary content type '%s': %s",
+					g_app_info_get_name (info), supported_mime_types[i], error->message);
+		else
+			g_debug ("Set '%s' as the default application for '%s'",
+					g_app_info_get_name (info),
+					supported_mime_types[i]);
+	}
+}
+
 void remmina_main_on_action_application_quit(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	// Called by quit signal in remmina_main.glade
 	TRACE_CALL(__func__);
-	g_print ("Quit intercept");
+	g_debug ("Quit intercept");
 	remmina_application_condexit(REMMINA_CONDEXIT_ONQUIT);
 }
 
@@ -853,6 +941,7 @@ static void remmina_main_import_file_list(GSList *files)
 	}
 	g_slist_free(files);
 	if (err->len > 0) {
+		// TRANSLATORS: The placeholder %s is an error message
 		dlg = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 			_("Unable to import:\n%s"), err->str);
 		g_signal_connect(G_OBJECT(dlg), "response", G_CALLBACK(gtk_widget_destroy), NULL);
@@ -927,7 +1016,7 @@ void remmina_main_on_action_application_plugins(GSimpleAction *action, GVariant
 void remmina_main_on_action_help_homepage(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	TRACE_CALL(__func__);
-	g_app_info_launch_default_for_uri("http://www.remmina.org", NULL, NULL);
+	g_app_info_launch_default_for_uri("https://www.remmina.org", NULL, NULL);
 }
 
 void remmina_main_on_action_help_wiki(GSimpleAction *action, GVariant *param, gpointer data)
@@ -945,7 +1034,7 @@ void remmina_main_on_action_help_community(GSimpleAction *action, GVariant *para
 void remmina_main_on_action_help_donations(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	TRACE_CALL(__func__);
-	g_app_info_launch_default_for_uri("http://www.remmina.org/wp/donations", NULL, NULL);
+	g_app_info_launch_default_for_uri("https://www.remmina.org/donations", NULL, NULL);
 }
 
 void remmina_main_on_action_help_debug(GSimpleAction *action, GVariant *param, gpointer data)
@@ -960,20 +1049,37 @@ void remmina_main_on_action_application_about(GSimpleAction *action, GVariant *p
 	remmina_about_open(remminamain->window);
 };
 
+void remmina_main_on_action_application_news(GSimpleAction *action, GVariant *param, gpointer data)
+{
+	TRACE_CALL(__func__);
+	remmina_pref.periodic_rmnews_last_get = 0;
+	remmina_pref.periodic_rmnews_get_count = 0;
+	remmina_pref_save();
+};
+
 static gboolean remmina_main_quickconnect(void)
 {
 	TRACE_CALL(__func__);
 	RemminaFile* remminafile;
 	gchar* server;
+	gchar *qcp;
+
+
+	/* Save quick connect protocol if different from the previuous one */
+	qcp = gtk_combo_box_text_get_active_text(remminamain->combo_quick_connect_protocol);
+	if (strcmp(qcp, remmina_pref.last_quickconnect_protocol) != 0) {
+		g_free(remmina_pref.last_quickconnect_protocol);
+		remmina_pref.last_quickconnect_protocol = g_strdup(qcp);
+		remmina_pref_save();
+	}
 
 	remminafile = remmina_file_new();
-	server = strdup(gtk_entry_get_text(remminamain->entry_quick_connect_server));
+	server = g_strdup(gtk_entry_get_text(remminamain->entry_quick_connect_server));
 
 	remmina_file_set_string(remminafile, "sound", "off");
 	remmina_file_set_string(remminafile, "server", server);
 	remmina_file_set_string(remminafile, "name", server);
-	remmina_file_set_string(remminafile, "protocol",
-		gtk_combo_box_text_get_active_text(remminamain->combo_quick_connect_protocol));
+	remmina_file_set_string(remminafile, "protocol", qcp);
 	g_free(server);
 
 	rcw_open_from_file(remminafile);
@@ -996,6 +1102,18 @@ void remmina_main_quick_search_enter(GtkWidget *widget, gpointer user_data)
 		gtk_editable_select_region(GTK_EDITABLE(remminamain->entry_quick_connect_server), 0, -1);
 }
 
+void remmina_main_on_action_collapse (GSimpleAction *action, GVariant *param, gpointer data)
+{
+	TRACE_CALL(__func__);
+	gtk_tree_view_collapse_all (remminamain->tree_files_list);
+}
+
+void remmina_main_on_action_expand (GSimpleAction *action, GVariant *param, gpointer data)
+{
+	TRACE_CALL(__func__);
+	gtk_tree_view_expand_all (remminamain->tree_files_list);
+}
+
 /* Handle double click on a row in the connections list */
 void remmina_main_file_list_on_row_activated(GtkTreeView *tree, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
 {
@@ -1116,7 +1234,7 @@ gboolean remmina_main_on_window_state_event(GtkWidget *widget, GdkEventWindowSta
 static void remmina_main_init(void)
 {
 	TRACE_CALL(__func__);
-	int i;
+	int i, qcp_idx, qcp_actidx;
 	char *name;
 
 	remminamain->priv->expanded_group = remmina_string_array_new_from_string(remmina_pref.expanded_group);
@@ -1134,18 +1252,23 @@ static void remmina_main_init(void)
 
 	/* Honor global peferences Search Bar visibility */
 	if (remmina_pref.hide_searchbar)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GET_OBJECT("search_toggle")), FALSE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(RM_GET_OBJECT("search_toggle")), FALSE);
 
 	/* Add a GtkMenuItem to the Tools menu for each plugin of type REMMINA_PLUGIN_TYPE_TOOL */
 	remmina_plugin_manager_for_each_plugin(REMMINA_PLUGIN_TYPE_TOOL, remmina_main_add_tool_plugin, remminamain);
 
 	/* Add available quick connect protocols to remminamain->combo_quick_connect_protocol */
+	qcp_idx = qcp_actidx = 0;
 	for (i = 0; i < sizeof(quick_connect_plugin_list) / sizeof(quick_connect_plugin_list[0]); i++) {
 		name = quick_connect_plugin_list[i];
-		if (remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, name))
+		if (remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, name)) {
 			gtk_combo_box_text_append(remminamain->combo_quick_connect_protocol, name, name);
+			if (remmina_pref.last_quickconnect_protocol != NULL && strcmp(name, remmina_pref.last_quickconnect_protocol) == 0)
+				qcp_actidx = qcp_idx;
+			qcp_idx ++;
+		}
 	}
-	gtk_combo_box_set_active(GTK_COMBO_BOX(remminamain->combo_quick_connect_protocol), 0);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(remminamain->combo_quick_connect_protocol), qcp_actidx);
 
 	/* Connect the group accelerators to the GtkWindow */
 	//gtk_window_add_accel_group(remminamain->window, remminamain->accelgroup_shortcuts);
@@ -1159,12 +1282,6 @@ static void remmina_main_init(void)
 	gtk_tree_view_set_search_entry(remminamain->tree_files_list, GTK_ENTRY(remminamain->entry_quick_connect_server));
 	/* Load the files list */
 	remmina_main_load_files();
-	/* Load the preferences */
-	if (remmina_pref.view_file_mode) {
-		G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-		gtk_toggle_action_set_active(remminamain->action_view_mode_tree, TRUE);
-		G_GNUC_END_IGNORE_DEPRECATIONS
-	}
 
 	/* Drag-n-drop support */
 	gtk_drag_dest_set(GTK_WIDGET(remminamain->window), GTK_DEST_DEFAULT_ALL, remmina_drop_types, 1, GDK_ACTION_COPY);
@@ -1181,35 +1298,12 @@ static void remmina_main_init(void)
 void remmina_main_on_show(GtkWidget *w, gpointer user_data)
 {
 	TRACE_CALL(__func__);
-	if (!remmina_pref.periodic_usage_stats_permission_asked) {
-		gtk_widget_set_visible(GTK_WIDGET(remminamain->box_ustat), TRUE);
-	}
-
 #ifdef SNAP_BUILD
 	remmina_main_show_snap_welcome();
 #endif
-
+	remmina_main_check_env();
 }
 
-void remmina_main_on_click_ustat_yes(GtkWidget *w, gpointer user_data)
-{
-	remmina_pref.periodic_usage_stats_permission_asked = TRUE;
-	remmina_pref.periodic_usage_stats_permitted = TRUE;
-	gtk_widget_set_visible(GTK_WIDGET(remminamain->box_ustat), FALSE);
-	if (remmina_pref_save()) {
-		remmina_stats_sender_schedule();
-	}
-}
-
-void remmina_main_on_click_ustat_no(GtkWidget *w, gpointer user_data)
-{
-	remmina_pref.periodic_usage_stats_permission_asked = TRUE;
-	remmina_pref.periodic_usage_stats_permitted = FALSE;
-	remmina_pref_save();
-	gtk_widget_set_visible(GTK_WIDGET(remminamain->box_ustat), FALSE);
-}
-
-
 /* RemminaMain instance */
 GtkWidget* remmina_main_new(void)
 {
@@ -1221,43 +1315,45 @@ GtkWidget* remmina_main_new(void)
 	remminamain->priv = g_new0(RemminaMainPriv, 1);
 	/* Assign UI widgets to the private members */
 	remminamain->builder = remmina_public_gtk_builder_new_from_file("remmina_main.glade");
-	remminamain->window = GTK_WINDOW(GET_OBJECT("RemminaMain"));
+	remminamain->window = GTK_WINDOW(RM_GET_OBJECT("RemminaMain"));
 	if (kioskmode && kioskmode == TRUE) {
 		gtk_window_set_position(remminamain->window, GTK_WIN_POS_CENTER_ALWAYS);
 		gtk_window_set_default_size(remminamain->window, 800, 400);
 		gtk_window_set_resizable(remminamain->window, FALSE);
 	}
 	/* New Button */
-	remminamain->button_new = GTK_BUTTON(GET_OBJECT("button_new"));
+	remminamain->button_new = GTK_BUTTON(RM_GET_OBJECT("button_new"));
 	if (kioskmode && kioskmode == TRUE)
 		gtk_widget_set_sensitive(GTK_WIDGET(remminamain->button_new), FALSE);
 	/* Search bar */
-	remminamain->search_toggle = GTK_TOGGLE_BUTTON(GET_OBJECT("search_toggle"));
-	remminamain->search_bar = GTK_SEARCH_BAR(GET_OBJECT("search_bar"));
-	remminamain->view_toggle_button = GTK_TOGGLE_BUTTON(GET_OBJECT("view_toggle_button"));
+	remminamain->search_toggle = GTK_TOGGLE_BUTTON(RM_GET_OBJECT("search_toggle"));
+	remminamain->search_bar = GTK_SEARCH_BAR(RM_GET_OBJECT("search_bar"));
+	/* view mode list/tree */
+	remminamain->view_toggle_button = GTK_TOGGLE_BUTTON(RM_GET_OBJECT("view_toggle_button"));
+	if (kioskmode && kioskmode == TRUE)
+		gtk_widget_set_sensitive(GTK_WIDGET(remminamain->view_toggle_button), FALSE);
+
 	/* Menu widgets */
-	remminamain->menu_popup = GTK_MENU(GET_OBJECT("menu_popup"));
-	remminamain->menu_header_button = GTK_MENU_BUTTON(GET_OBJECT("menu_header_button"));
-	remminamain->menu_popup_full = GTK_MENU(GET_OBJECT("menu_popup_full"));
+	remminamain->menu_popup = GTK_MENU(RM_GET_OBJECT("menu_popup"));
+	remminamain->menu_header_button = GTK_MENU_BUTTON(RM_GET_OBJECT("menu_header_button"));
+	remminamain->menu_popup_full = GTK_MENU(RM_GET_OBJECT("menu_popup_full"));
 	if (kioskmode && kioskmode == TRUE) {
 		gtk_widget_set_sensitive(GTK_WIDGET(remminamain->menu_popup_full), FALSE);
 		gtk_widget_set_sensitive(GTK_WIDGET(remminamain->menu_header_button), FALSE);
 	}
 	/* View mode radios */
-	remminamain->menuitem_view_mode_list = GTK_RADIO_MENU_ITEM(GET_OBJECT("menuitem_view_mode_list"));
-	remminamain->menuitem_view_mode_tree = GTK_RADIO_MENU_ITEM(GET_OBJECT("menuitem_view_mode_tree"));
+	remminamain->menuitem_view_mode_list = GTK_RADIO_MENU_ITEM(RM_GET_OBJECT("menuitem_view_mode_list"));
+	remminamain->menuitem_view_mode_tree = GTK_RADIO_MENU_ITEM(RM_GET_OBJECT("menuitem_view_mode_tree"));
 	/* Quick connect objects */
-	remminamain->box_quick_connect = GTK_BOX(GET_OBJECT("box_quick_connect"));
-	remminamain->combo_quick_connect_protocol = GTK_COMBO_BOX_TEXT(GET_OBJECT("combo_quick_connect_protocol"));
+	remminamain->box_quick_connect = GTK_BOX(RM_GET_OBJECT("box_quick_connect"));
+	remminamain->combo_quick_connect_protocol = GTK_COMBO_BOX_TEXT(RM_GET_OBJECT("combo_quick_connect_protocol"));
 	if (kioskmode && kioskmode == TRUE)
 		gtk_widget_set_sensitive(GTK_WIDGET(remminamain->combo_quick_connect_protocol), FALSE);
-	remminamain->entry_quick_connect_server = GTK_ENTRY(GET_OBJECT("entry_quick_connect_server"));
+	remminamain->entry_quick_connect_server = GTK_ENTRY(RM_GET_OBJECT("entry_quick_connect_server"));
 	/* Other widgets */
-	remminamain->tree_files_list = GTK_TREE_VIEW(GET_OBJECT("tree_files_list"));
-	remminamain->column_files_list_group = GTK_TREE_VIEW_COLUMN(GET_OBJECT("column_files_list_group"));
-	remminamain->statusbar_main = GTK_STATUSBAR(GET_OBJECT("statusbar_main"));
-	if (!kioskmode && kioskmode == FALSE)
-		remminamain->box_ustat = GTK_BOX(GET_OBJECT("box_ustat"));
+	remminamain->tree_files_list = GTK_TREE_VIEW(RM_GET_OBJECT("tree_files_list"));
+	remminamain->column_files_list_group = GTK_TREE_VIEW_COLUMN(RM_GET_OBJECT("column_files_list_group"));
+	remminamain->statusbar_main = GTK_STATUSBAR(RM_GET_OBJECT("statusbar_main"));
 	/* Non widget objects */
 	actions = g_simple_action_group_new();
 	g_action_map_add_action_entries(G_ACTION_MAP(actions), main_actions, G_N_ELEMENTS(main_actions), remminamain->window);
@@ -1286,6 +1382,7 @@ GtkWindow* remmina_main_get_window()
 		return NULL;
 	if (!remminamain->priv->initialized)
 		return NULL;
+	remminamain->window = GTK_WINDOW(RM_GET_OBJECT("RemminaMain"));
 	return remminamain->window;
 }
 
