@@ -101,6 +101,7 @@ static void init_settings_cache(RemminaPlugin *plugin)
 	}
 
 	/* Some settings are encrypted "by name" */
+	/* g_hash_table_insert(pht, g_strdup("proxy_password"), (gpointer)TRUE); */
 	g_hash_table_insert(pht, g_strdup("ssh_password"), (gpointer)TRUE);
 	g_hash_table_insert(pht, g_strdup("ssh_passphrase"), (gpointer)TRUE);
 
@@ -129,12 +130,10 @@ static gboolean remmina_plugin_manager_register_plugin(RemminaPlugin *plugin)
 {
 	TRACE_CALL(__func__);
 	if (plugin->type == REMMINA_PLUGIN_TYPE_SECRET) {
-		if (remmina_secret_plugin) {
-			g_print("Remmina plugin %s (type=%s) bypassed.\n", plugin->name,
-				_(remmina_plugin_type_name[plugin->type]));
-			return FALSE;
-		}
-		remmina_secret_plugin = (RemminaSecretPlugin*)plugin;
+		g_print("Remmina plugin %s (type=%s) has registered but not yet initialized/activated. "
+			"Initialization order is %d.\n", plugin->name,
+			_(remmina_plugin_type_name[plugin->type]),
+			((RemminaSecretPlugin*)plugin)->init_order);
 	}
 	init_settings_cache(plugin);
 
@@ -155,7 +154,7 @@ gboolean remmina_gtksocket_available()
 
 #ifdef GDK_WINDOWING_X11
 	if (GDK_IS_X11_DISPLAY(d)) {
-		/* GtkSocket support is available only under Xorg */
+		/* GtkSocket support is available only under X.Org */
 		available = TRUE;
 	}
 #endif
@@ -184,9 +183,12 @@ RemminaPluginService remmina_plugin_manager_service =
 	remmina_protocol_widget_start_reverse_tunnel,
 	remmina_protocol_widget_start_xport_tunnel,
 	remmina_protocol_widget_set_display,
-	remmina_protocol_widget_close_connection,
-	remmina_protocol_widget_panel_authpwd,
-	remmina_protocol_widget_panel_authuserpwd,
+	remmina_protocol_widget_signal_connection_closed,
+    remmina_protocol_widget_signal_connection_opened,
+    remmina_protocol_widget_update_align,
+    remmina_protocol_widget_unlock_dynres,
+    remmina_protocol_widget_desktop_resize,
+	remmina_protocol_widget_panel_auth,
 	remmina_protocol_widget_panel_new_certificate,
 	remmina_protocol_widget_panel_changed_certificate,
 	remmina_protocol_widget_get_username,
@@ -218,7 +220,7 @@ RemminaPluginService remmina_plugin_manager_service =
 	remmina_file_get_secret,
 	remmina_file_set_int,
 	remmina_file_get_int,
-	remmina_file_unsave_password,
+	remmina_file_unsave_passwords,
 
 	remmina_pref_set_value,
 	remmina_pref_get_value,
@@ -269,12 +271,31 @@ static void remmina_plugin_manager_load_plugin(const gchar *name)
 	/* We don’t close the module because we will need it throughout the process lifetime */
 }
 
+static gint compare_secret_plugin_init_order(gconstpointer a, gconstpointer b)
+{
+	RemminaSecretPlugin *sa, *sb;
+
+	sa = (RemminaSecretPlugin*)a;
+	sb = (RemminaSecretPlugin*)b;
+
+	if (sa->init_order > sb->init_order)
+		return 1;
+	else if (sa->init_order < sb->init_order)
+		return -1;
+	return 0;
+}
+
 void remmina_plugin_manager_init(void)
 {
 	TRACE_CALL(__func__);
 	GDir *dir;
 	const gchar *name, *ptr;
 	gchar *fullpath;
+	RemminaPlugin *plugin;
+	RemminaSecretPlugin *sp;
+	int i;
+	GSList *secret_plugins;
+	GSList *sple;
 
 	remmina_plugin_table = g_ptr_array_new();
 
@@ -297,6 +318,35 @@ void remmina_plugin_manager_init(void)
 		g_free(fullpath);
 	}
 	g_dir_close(dir);
+
+	/* Now all secret plugins needs to initialize, following their init_order.
+	 * The 1st plugin which will initialize correctly will be
+	 * the default remmina_secret_plugin */
+
+	if (remmina_secret_plugin)
+		g_print("Internal ERROR: remmina_secret_plugin must be null here\n");
+
+	secret_plugins = NULL;
+	for (i = 0; i < remmina_plugin_table->len; i++) {
+		plugin = (RemminaPlugin*)g_ptr_array_index(remmina_plugin_table, i);
+		if (plugin->type == REMMINA_PLUGIN_TYPE_SECRET) {
+			secret_plugins = g_slist_insert_sorted(secret_plugins, (gpointer)plugin, compare_secret_plugin_init_order);
+		}
+	}
+
+	sple = secret_plugins;
+	while(sple != NULL) {
+		sp = (RemminaSecretPlugin*)sple->data;
+		if (sp->init()) {
+			g_print("Secret plugin %s has been successfully initialized and will be your default secret plugin\n",
+				sp->name);
+			remmina_secret_plugin = sp;
+			break;
+		}
+		sple = sple->next;
+	}
+
+	g_slist_free(secret_plugins);
 }
 
 RemminaPlugin* remmina_plugin_manager_get_plugin(RemminaPluginType type, const gchar *name)
@@ -508,7 +558,3 @@ gboolean remmina_plugin_manager_is_encrypted_setting(RemminaProtocolPlugin *pp,
 
 	return TRUE;
 }
-
-
-
-
