@@ -55,48 +55,53 @@ G_BEGIN_DECLS
 typedef struct _RemminaProtocolWidgetPriv RemminaProtocolWidgetPriv;
 
 struct _RemminaProtocolWidget {
-	GtkEventBox event_box;
-	RemminaConnectionObject *cnnobj;
-	RemminaProtocolWidgetPriv *priv;
+	GtkEventBox			event_box;
+	RemminaConnectionObject *	cnnobj;
+	RemminaProtocolWidgetPriv *	priv;
 };
 
 struct _RemminaProtocolWidgetClass {
-	GtkEventBoxClass parent_class;
+	GtkEventBoxClass	parent_class;
 
-	void (*connect)(RemminaProtocolWidget *gp);
-	void (*disconnect)(RemminaProtocolWidget *gp);
-	void (*desktop_resize)(RemminaProtocolWidget *gp);
-	void (*update_align)(RemminaProtocolWidget *gp);
-	void (*unlock_dynres)(RemminaProtocolWidget *gp);
+	void			(*connect)(RemminaProtocolWidget *gp);
+	void			(*disconnect)(RemminaProtocolWidget *gp);
+	void			(*desktop_resize)(RemminaProtocolWidget *gp);
+	void			(*update_align)(RemminaProtocolWidget *gp);
+	void			(*unlock_dynres)(RemminaProtocolWidget *gp);
 };
 
 GType remmina_protocol_widget_get_type(void)
 G_GNUC_CONST;
 
-GtkWidget* remmina_protocol_widget_new(void);
-void remmina_protocol_widget_setup(RemminaProtocolWidget *gp, RemminaFile* remminafile, RemminaConnectionObject* cnnobj);
+GtkWidget *remmina_protocol_widget_new(void);
+void remmina_protocol_widget_setup(RemminaProtocolWidget *gp, RemminaFile *remminafile, RemminaConnectionObject *cnnobj);
 
 gint remmina_protocol_widget_get_width(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_set_width(RemminaProtocolWidget *gp, gint width);
 gint remmina_protocol_widget_get_height(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_set_height(RemminaProtocolWidget *gp, gint height);
-gint remmina_protocol_widget_get_profile_remote_width(RemminaProtocolWidget* gp);
-gint remmina_protocol_widget_get_profile_remote_height(RemminaProtocolWidget* gp);
+gint remmina_protocol_widget_get_profile_remote_width(RemminaProtocolWidget *gp);
+gint remmina_protocol_widget_get_profile_remote_height(RemminaProtocolWidget *gp);
 
 RemminaScaleMode remmina_protocol_widget_get_current_scale_mode(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_set_current_scale_mode(RemminaProtocolWidget *gp, RemminaScaleMode scalemode);
 gboolean remmina_protocol_widget_get_expand(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_set_expand(RemminaProtocolWidget *gp, gboolean expand);
 gboolean remmina_protocol_widget_has_error(RemminaProtocolWidget *gp);
-const gchar* remmina_protocol_widget_get_error_message(RemminaProtocolWidget *gp);
+const gchar *remmina_protocol_widget_get_error_message(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_set_error(RemminaProtocolWidget *gp, const gchar *fmt, ...);
 gboolean remmina_protocol_widget_is_closed(RemminaProtocolWidget *gp);
-RemminaFile* remmina_protocol_widget_get_file(RemminaProtocolWidget *gp);
+RemminaFile *remmina_protocol_widget_get_file(RemminaProtocolWidget *gp);
 
 void remmina_protocol_widget_open_connection(RemminaProtocolWidget *gp);
-gboolean remmina_protocol_widget_close_connection(RemminaProtocolWidget *gp);
+void remmina_protocol_widget_close_connection(RemminaProtocolWidget *gp);
+void remmina_protocol_widget_signal_connection_closed(RemminaProtocolWidget *gp);
+void remmina_protocol_widget_signal_connection_opened(RemminaProtocolWidget *gp);
+void remmina_protocol_widget_update_align(RemminaProtocolWidget *gp);
+void remmina_protocol_widget_unlock_dynres(RemminaProtocolWidget *gp);
+void remmina_protocol_widget_desktop_resize(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_grab_focus(RemminaProtocolWidget *gp);
-const RemminaProtocolFeature* remmina_protocol_widget_get_features(RemminaProtocolWidget *gp);
+const RemminaProtocolFeature *remmina_protocol_widget_get_features(RemminaProtocolWidget *gp);
 gboolean remmina_protocol_widget_query_feature_by_type(RemminaProtocolWidget *gp, RemminaProtocolFeatureType type);
 gboolean remmina_protocol_widget_query_feature_by_ref(RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature);
 void remmina_protocol_widget_call_feature_by_type(RemminaProtocolWidget *gp, RemminaProtocolFeatureType type, gint id);
@@ -114,22 +119,21 @@ gboolean remmina_protocol_widget_ssh_exec(RemminaProtocolWidget *gp, gboolean wa
  * 1. The actual destination (host:port) if SSH tunnel is disable
  * 2. The tunnel local destination (127.0.0.1:port) if SSH tunnel is enabled
  */
-gchar* remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gint default_port, gboolean port_plus);
+gchar *remmina_protocol_widget_start_direct_tunnel(RemminaProtocolWidget *gp, gint default_port, gboolean port_plus);
 
 gboolean remmina_protocol_widget_start_reverse_tunnel(RemminaProtocolWidget *gp, gint local_port);
 gboolean remmina_protocol_widget_start_xport_tunnel(RemminaProtocolWidget *gp, RemminaXPortTunnelInitFunc init_func);
 void remmina_protocol_widget_set_display(RemminaProtocolWidget *gp, gint display);
 
 /* Extension for remmina_protocol_widget_panel_authuserpwd() not currently exported to plugins */
-gint remmina_protocol_widget_panel_authuserpwd_ssh_tunnel(RemminaProtocolWidget* gp, gboolean want_domain, gboolean allow_password_saving);
+gint remmina_protocol_widget_panel_authuserpwd_ssh_tunnel(RemminaProtocolWidget *gp, gboolean want_domain, gboolean allow_password_saving);
 
 /* Dialog panel API used by the plugins */
 
-gint remmina_protocol_widget_panel_authpwd(RemminaProtocolWidget *gp, RemminaAuthpwdType authpwd_type, gboolean allow_password_saving);
-gint remmina_protocol_widget_panel_authuserpwd(RemminaProtocolWidget *gp, gboolean want_domain, gboolean allow_password_saving);
-gint remmina_protocol_widget_panel_new_certificate(RemminaProtocolWidget* gp, const gchar* subject, const gchar* issuer, const gchar* fingerprint);
-gint remmina_protocol_widget_panel_changed_certificate(RemminaProtocolWidget *gp, const gchar* subject, const gchar* issuer, const gchar* new_fingerprint, const gchar* old_fingerprint);
-gint remmina_protocol_widget_panel_question_yesno(RemminaProtocolWidget* gp, const char *msg);
+gint remmina_protocol_widget_panel_auth(RemminaProtocolWidget *gp, RemminaMessagePanelFlags pflags, const gchar *title, const gchar *default_username, const gchar *default_password, const gchar *default_domain, const gchar *password_prompt);
+gint remmina_protocol_widget_panel_new_certificate(RemminaProtocolWidget *gp, const gchar *subject, const gchar *issuer, const gchar *fingerprint);
+gint remmina_protocol_widget_panel_changed_certificate(RemminaProtocolWidget *gp, const gchar *subject, const gchar *issuer, const gchar *new_fingerprint, const gchar *old_fingerprint);
+gint remmina_protocol_widget_panel_question_yesno(RemminaProtocolWidget *gp, const char *msg);
 
 void remmina_protocol_widget_panel_show(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_panel_hide(RemminaProtocolWidget *gp);
@@ -140,34 +144,31 @@ void remmina_protocol_widget_panel_show_retry(RemminaProtocolWidget *gp);
 
 void remmina_protocol_widget_save_cred(RemminaProtocolWidget *gp);
 
-gchar* remmina_protocol_widget_get_username(RemminaProtocolWidget *gp);
-gchar* remmina_protocol_widget_get_password(RemminaProtocolWidget *gp);
-gchar* remmina_protocol_widget_get_domain(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_username(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_password(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_domain(RemminaProtocolWidget *gp);
 gboolean remmina_protocol_widget_get_savepassword(RemminaProtocolWidget *gp);
-gchar* remmina_protocol_widget_get_cacert(RemminaProtocolWidget *gp);
-gchar* remmina_protocol_widget_get_cacrl(RemminaProtocolWidget *gp);
-gchar* remmina_protocol_widget_get_clientcert(RemminaProtocolWidget *gp);
-gchar* remmina_protocol_widget_get_clientkey(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_cacert(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_cacrl(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_clientcert(RemminaProtocolWidget *gp);
+gchar *remmina_protocol_widget_get_clientkey(RemminaProtocolWidget *gp);
 
-void remmina_protocol_widget_chat_open(RemminaProtocolWidget *gp, const gchar *name,
-				       void (*on_send)(RemminaProtocolWidget *gp, const gchar *text), void (*on_destroy)(RemminaProtocolWidget *gp));
+void remmina_protocol_widget_chat_open(RemminaProtocolWidget *gp, const gchar *name, void (*on_send)(RemminaProtocolWidget *gp, const gchar *text), void (*on_destroy)(RemminaProtocolWidget *gp));
 void remmina_protocol_widget_chat_close(RemminaProtocolWidget *gp);
 void remmina_protocol_widget_chat_receive(RemminaProtocolWidget *gp, const gchar *text);
 void remmina_protocol_widget_send_keys_signals(GtkWidget *widget, const guint *keyvals, int length, GdkEventType action);
 /* Check if the plugin accepts keystrokes */
-gboolean remmina_protocol_widget_plugin_receives_keystrokes(RemminaProtocolWidget* gp);
+gboolean remmina_protocol_widget_plugin_receives_keystrokes(RemminaProtocolWidget *gp);
 /* Send to the plugin some keystrokes */
-void remmina_protocol_widget_send_keystrokes(RemminaProtocolWidget* gp, GtkMenuItem *widget);
+void remmina_protocol_widget_send_keystrokes(RemminaProtocolWidget *gp, GtkMenuItem *widget);
 /* Take screenshot of plugin */
-gboolean remmina_protocol_widget_plugin_screenshot(RemminaProtocolWidget* gp, RemminaPluginScreenshotData *rpsd);
+gboolean remmina_protocol_widget_plugin_screenshot(RemminaProtocolWidget *gp, RemminaPluginScreenshotData *rpsd);
 
-void remmina_protocol_widget_update_remote_resolution(RemminaProtocolWidget* gp);
+void remmina_protocol_widget_update_remote_resolution(RemminaProtocolWidget *gp);
 
 /* Functions to support execution of GTK code on master thread */
-RemminaMessagePanel* remmina_protocol_widget_mpprogress(RemminaConnectionObject* cnnobj, const gchar *msg, RemminaMessagePanelCallback response_callback, gpointer response_callback_data);
+RemminaMessagePanel *remmina_protocol_widget_mpprogress(RemminaConnectionObject *cnnobj, const gchar *msg, RemminaMessagePanelCallback response_callback, gpointer response_callback_data);
 void remmina_protocol_widget_mpdestroy(RemminaConnectionObject *cnnobj, RemminaMessagePanel *mp);
 
 
 G_END_DECLS
-
-
