@@ -53,12 +53,12 @@
 
 #define REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY "overwrite_all"
 
-#define GET_PLUGIN_DATA(gp) (RemminaPluginSftpData*)g_object_get_data(G_OBJECT(gp), "plugin-data");
+#define GET_PLUGIN_DATA(gp) (RemminaPluginSftpData *)g_object_get_data(G_OBJECT(gp), "plugin-data");
 
 typedef struct _RemminaPluginSftpData {
-	GtkWidget *client;
-	pthread_t thread;
-	RemminaSFTP *sftp;
+	GtkWidget *	client;
+	pthread_t	thread;
+	RemminaSFTP *	sftp;
 } RemminaPluginSftpData;
 
 static RemminaPluginService *remmina_plugin_service = NULL;
@@ -67,7 +67,7 @@ static gpointer
 remmina_plugin_sftp_main_thread(gpointer data)
 {
 	TRACE_CALL(__func__);
-	RemminaProtocolWidget *gp = (RemminaProtocolWidget*)data;
+	RemminaProtocolWidget *gp = (RemminaProtocolWidget *)data;
 	RemminaPluginSftpData *gpdata;
 	RemminaFile *remminafile;
 	RemminaSSH *ssh;
@@ -78,16 +78,16 @@ remmina_plugin_sftp_main_thread(gpointer data)
 	const gchar *saveserver;
 	const gchar *saveusername;
 	gchar *hostport;
-	gchar tunneluser[33]; /* On linux a username can have a 32 char lenght */
-	gchar tunnelserver[256]; /* On linux a servername can have a 255 char lenght */
-	gchar tunnelport[6]; /* A TCP port can have a maximum value of 65535 */
+	gchar tunneluser[33];           /* On linux a username can have a 32 char lenght */
+	gchar tunnelserver[256];        /* On linux a servername can have a 255 char lenght */
+	gchar tunnelport[6];            /* A TCP port can have a maximum value of 65535 */
 	gchar *host;
 	gint port;
 
 	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	CANCEL_ASYNC
 
-	gpdata = GET_PLUGIN_DATA(gp);
+		gpdata = GET_PLUGIN_DATA(gp);
 
 	/* remmina_plugin_service->protocol_plugin_start_direct_tunnel start the
 	 * SSH Tunnel and return the server + port string
@@ -109,25 +109,23 @@ remmina_plugin_sftp_main_thread(gpointer data)
 		 * */
 		if (strchr(saveserver, '@')) {
 			sscanf(saveserver, "%[_a-zA-Z0-9.]@%[_a-zA-Z0-9.]:%[0-9]",
-					tunneluser, tunnelserver, tunnelport);
-			g_print ("Username: %s, tunneluser: %s\n",
-					remmina_plugin_service->file_get_string(remminafile, "ssh_username"), tunneluser);
-			if (saveusername != NULL && tunneluser[0] != '\0') {
+			       tunneluser, tunnelserver, tunnelport);
+			g_print("Username: %s, tunneluser: %s\n",
+				remmina_plugin_service->file_get_string(remminafile, "ssh_username"), tunneluser);
+			if (saveusername != NULL && tunneluser[0] != '\0')
 				remmina_plugin_service->file_set_string(remminafile, "ssh_username", NULL);
-			}
 			remmina_plugin_service->file_set_string(remminafile, "ssh_username",
-					g_strdup(tunneluser));
+								g_strdup(tunneluser));
 			remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-					g_strconcat (tunnelserver, ":", tunnelport, NULL));
+								g_strconcat(tunnelserver, ":", tunnelport, NULL));
 		}
-        }
+	}
 
 	hostport = remmina_plugin_service->protocol_plugin_start_direct_tunnel(gp, 22, FALSE);
 	/* We restore the SSH username as the tunnel is set */
 	remmina_plugin_service->file_set_string(remminafile, "ssh_username", g_strdup(saveusername));
-	if (hostport == NULL) {
+	if (hostport == NULL)
 		return FALSE;
-	}
 	remmina_plugin_service->get_server_port(hostport, 22, &host, &port);
 
 	ssh = g_object_get_data(G_OBJECT(gp), "user-data");
@@ -135,18 +133,16 @@ remmina_plugin_sftp_main_thread(gpointer data)
 		/* Create SFTP connection based on existing SSH session */
 		sftp = remmina_sftp_new_from_ssh(ssh);
 		if (remmina_ssh_init_session(REMMINA_SSH(sftp)) &&
-		    remmina_ssh_auth(REMMINA_SSH(sftp), NULL) > 0 &&
-		    remmina_sftp_open(sftp)) {
+		    remmina_ssh_auth(REMMINA_SSH(sftp), NULL, gp, remminafile) > 0 &&
+		    remmina_sftp_open(sftp))
 			cont = TRUE;
-		}
-	}else {
+	} else {
 		/* New SFTP connection */
-		if (remmina_plugin_service->file_get_int(remminafile, "ssh_enabled", FALSE)) {
+		if (remmina_plugin_service->file_get_int(remminafile, "ssh_enabled", FALSE))
 			remmina_plugin_service->file_set_string(remminafile, "ssh_server", g_strdup(hostport));
-		}else {
+		else
 			remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-				remmina_plugin_service->file_get_string(remminafile, "server"));
-		}
+								remmina_plugin_service->file_get_string(remminafile, "server"));
 		g_free(hostport);
 		g_free(host);
 
@@ -158,9 +154,8 @@ remmina_plugin_sftp_main_thread(gpointer data)
 			}
 
 			ret = remmina_ssh_auth_gui(REMMINA_SSH(sftp), gp, remminafile);
-			if (ret == 0) {
+			if (ret == 0)
 				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(sftp)->error);
-			}
 			if (ret <= 0) break;
 
 			if (!remmina_sftp_open(sftp)) {
@@ -169,9 +164,8 @@ remmina_plugin_sftp_main_thread(gpointer data)
 			}
 
 			cs = remmina_plugin_service->file_get_string(remminafile, "execpath");
-			if (cs && cs[0]) {
+			if (cs && cs[0])
 				remmina_ftp_client_set_dir(REMMINA_FTP_CLIENT(gpdata->client), cs);
-			}
 
 			cont = TRUE;
 			break;
@@ -179,11 +173,11 @@ remmina_plugin_sftp_main_thread(gpointer data)
 
 		/* We restore the ssh_server name */
 		remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-				remmina_plugin_service->file_get_string(remminafile, "save_ssh_server"));
+							remmina_plugin_service->file_get_string(remminafile, "save_ssh_server"));
 	}
 	if (!cont) {
 		if (sftp) remmina_sftp_free(sftp);
-		IDLE_ADD((GSourceFunc)remmina_plugin_service->protocol_plugin_close_connection, gp);
+		remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 		return NULL;
 	}
 
@@ -191,7 +185,7 @@ remmina_plugin_sftp_main_thread(gpointer data)
 	/* RemminaSFTPClient owns the object, we just take the reference */
 	gpdata->sftp = sftp;
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "connect");
+	remmina_plugin_service->protocol_plugin_signal_connection_opened(gp);
 
 	gpdata->thread = 0;
 	return NULL;
@@ -224,16 +218,16 @@ remmina_plugin_sftp_init(RemminaProtocolWidget *gp)
 	gtk_container_add(GTK_CONTAINER(gp), gpdata->client);
 
 	remmina_ftp_client_set_show_hidden(REMMINA_FTP_CLIENT(gpdata->client),
-		remmina_plugin_service->file_get_int(remminafile, "showhidden", FALSE));
+					   remmina_plugin_service->file_get_int(remminafile, "showhidden", FALSE));
 
 	remmina_ftp_client_set_overwrite_status(REMMINA_FTP_CLIENT(gpdata->client),
-		remmina_plugin_service->file_get_int(remminafile,
-			REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE));
+						remmina_plugin_service->file_get_int(remminafile,
+										     REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE));
 
 	remmina_plugin_service->protocol_plugin_register_hostkey(gp, gpdata->client);
 
 	g_signal_connect(G_OBJECT(gpdata->client),
-		"realize", G_CALLBACK(remmina_plugin_sftp_client_on_realize), gp);
+			 "realize", G_CALLBACK(remmina_plugin_sftp_client_on_realize), gp);
 }
 
 static gboolean
@@ -248,10 +242,10 @@ remmina_plugin_sftp_open_connection(RemminaProtocolWidget *gp)
 
 	if (pthread_create(&gpdata->thread, NULL, remmina_plugin_sftp_main_thread, gp)) {
 		remmina_plugin_service->protocol_plugin_set_error(gp,
-			"Failed to initialize pthread. Falling back to non-thread mode…");
+								  "Could not initialize pthread. Falling back to non-thread mode…");
 		gpdata->thread = 0;
 		return FALSE;
-	}else {
+	} else {
 		return TRUE;
 	}
 	return TRUE;
@@ -271,12 +265,12 @@ remmina_plugin_sftp_close_connection(RemminaProtocolWidget *gp)
 	}
 
 	remmina_ftp_client_save_state(REMMINA_FTP_CLIENT(gpdata->client), remminafile);
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "disconnect");
+	remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 	/* The session preference overwrite_all is always saved to FALSE in order
 	 * to avoid unwanted overwriting.
 	 * If we'd change idea just remove the next line to save the preference. */
 	remmina_file_set_int(remminafile,
-		REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE);
+			     REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE);
 	return FALSE;
 }
 
@@ -308,12 +302,12 @@ remmina_plugin_sftp_call_feature(RemminaProtocolWidget *gp, const RemminaProtoco
 		return;
 	case REMMINA_PLUGIN_SFTP_FEATURE_PREF_SHOW_HIDDEN:
 		remmina_ftp_client_set_show_hidden(REMMINA_FTP_CLIENT(gpdata->client),
-			remmina_plugin_service->file_get_int(remminafile, "showhidden", FALSE));
+						   remmina_plugin_service->file_get_int(remminafile, "showhidden", FALSE));
 		return;
 	case REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL:
 		remmina_ftp_client_set_overwrite_status(REMMINA_FTP_CLIENT(gpdata->client),
-			remmina_plugin_service->file_get_int(remminafile,
-				REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE));
+							remmina_plugin_service->file_get_int(remminafile,
+											     REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, FALSE));
 		return;
 	}
 }
@@ -332,12 +326,12 @@ static gpointer ssh_auth[] =
  * The last element of the array must be REMMINA_PROTOCOL_FEATURE_TYPE_END. */
 static const RemminaProtocolFeature remmina_plugin_sftp_features[] =
 {
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_SFTP_FEATURE_PREF_SHOW_HIDDEN,   GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK),   "showhidden",
-	  N_("Show Hidden Files")						},
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_SFTP_FEATURE_PREF_SHOW_HIDDEN,   GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "showhidden",
+	  N_("Show Hidden Files") },
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL, GINT_TO_POINTER(REMMINA_PROTOCOL_FEATURE_PREF_CHECK),
-	  REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY,	 N_("Overwrite all")													},
-	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,  0,					      NULL,						      NULL,
-		NULL														       }
+	  REMMINA_PLUGIN_SFTP_FEATURE_PREF_OVERWRITE_ALL_KEY, N_("Overwrite all") },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,  0,					      NULL,						    NULL,
+	  NULL }
 };
 
 /* Array of RemminaProtocolSetting for basic settings.
@@ -347,17 +341,17 @@ static const RemminaProtocolFeature remmina_plugin_sftp_features[] =
  * c) Setting description
  * d) Compact disposition
  * e) Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO
- * f) Unused pointer
+ * f) Setting Tooltip
  */
 static const RemminaProtocolSetting remmina_sftp_basic_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,   "ssh_server",	    NULL,			  FALSE, "_sftp-ssh._tcp",  NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "ssh_username",   N_("User name"),		  FALSE, NULL,	   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_password",   N_("User password"),	  FALSE, NULL,	   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "ssh_auth",	    N_("Authentication type"),	  FALSE, ssh_auth, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_FILE,	  "ssh_privatekey", N_("Identity file"),	  FALSE, NULL,	   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_passphrase", N_("Private key passphrase"), FALSE, NULL,	   NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		    NULL,			  FALSE, NULL,	   NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "ssh_server",	    NULL,			  FALSE, "_sftp-ssh._tcp", NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "ssh_username",   N_("Username"),		  FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_password",   N_("Password"),	  FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "ssh_auth",	    N_("Authentication type"),	  FALSE, ssh_auth,	   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_FILE,	  "ssh_privatekey", N_("Identity file"),	  FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_passphrase", N_("Password to unlock private key"), FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		    NULL,			  FALSE, NULL,		   NULL }
 };
 
 /* Protocol plugin definition and features */
@@ -380,7 +374,7 @@ static RemminaProtocolPlugin remmina_plugin_sftp =
 	remmina_plugin_sftp_query_feature,              // Query for available features
 	remmina_plugin_sftp_call_feature,               // Call a feature
 	NULL,                                           // Send a keystroke
-	NULL                                            // No screenshot support available
+	NULL                                            // Screenshot support unavailable
 };
 
 void
@@ -388,7 +382,7 @@ remmina_sftp_plugin_register(void)
 {
 	TRACE_CALL(__func__);
 	remmina_plugin_service = &remmina_plugin_manager_service;
-	remmina_plugin_service->register_plugin((RemminaPlugin*)&remmina_plugin_sftp);
+	remmina_plugin_service->register_plugin((RemminaPlugin *)&remmina_plugin_sftp);
 }
 
 #else
@@ -399,4 +393,3 @@ void remmina_sftp_plugin_register(void)
 }
 
 #endif
-
