@@ -81,6 +81,7 @@
  *    }
  *    "APPINDICATOR": {
  *        "appindicator_supported": 0
+ *        "appindicator_compiled": 1
  *        "icon_is_active": 1
  *        "appindicator_type": "AppIndicator on GtkStatusIcon/xembed"
  *    }
@@ -103,6 +104,15 @@
  *        "DATE_PYTHON_SIMPLE": ""
  *        "DATE_SPICE": ""
  *    }
+ *    "ENVIRONMENT": {
+ *        "language": "en_US.utf8"
+ *    }
+ *    "ACTIVESECRETPLUGIN": {
+ *        "plugin_name": "kwallet"
+ *    }
+ *    "HASMASTERPASSWORD": {
+ *        "master_password_status": "OFF"
+ *    }
  *
  * }
  * @endcode
@@ -122,7 +132,6 @@
 
 
 #include "config.h"
-#include <locale.h>
 #include <string.h>
 #include <sys/utsname.h>
 #include <unistd.h>
@@ -131,6 +140,7 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
+#include "remmina.h"
 #include "remmina_file.h"
 #include "remmina_file_manager.h"
 #include "remmina_icon.h"
@@ -139,6 +149,7 @@
 #include "remmina_sysinfo.h"
 #include "remmina_utils.h"
 #include "remmina/remmina_trace_calls.h"
+#include "remmina_plugin_manager.h"
 
 #ifdef GDK_WINDOWING_WAYLAND
 	#include <gdk/gdkwayland.h>
@@ -159,33 +170,6 @@ struct ProfilesData {
 	gchar datestr;
 };
 
-static gchar* remmina_stats_gen_random_uuid_prefix()
-{
-	TRACE_CALL(__func__);
-	GRand *rand;
-	GTimeVal t;
-	gchar *result;
-	int i;
-	static char alpha[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
-
-	result = g_malloc0(15);
-
-	g_get_current_time(&t);
-	rand = g_rand_new_with_seed((guint32)t.tv_sec ^ (guint32)t.tv_usec);
-
-	for (i = 0; i < 7; i++) {
-		result[i] = alpha[g_rand_int_range(rand, 0, sizeof(alpha) - 1)];
-	}
-
-	g_rand_set_seed(rand, (guint32)t.tv_usec);
-	for (i = 0; i < 7; i++) {
-		result[i + 7] = alpha[g_rand_int_range(rand, 0, sizeof(alpha) - 1)];
-	}
-	g_rand_free(rand);
-
-	return result;
-}
-
 JsonNode *remmina_stats_get_uid()
 {
 	TRACE_CALL(__func__);
@@ -202,7 +186,7 @@ JsonNode *remmina_stats_get_uid()
 
 	if (remmina_pref.periodic_usage_stats_uuid_prefix == NULL || remmina_pref.periodic_usage_stats_uuid_prefix[0] == 0) {
 		/* Generate a new UUID_PREFIX for this installation */
-		uid_prefix = remmina_stats_gen_random_uuid_prefix();
+		uid_prefix = remmina_gen_random_uuid();
 		if (remmina_pref.periodic_usage_stats_uuid_prefix)
 			g_free(remmina_pref.periodic_usage_stats_uuid_prefix);
 		remmina_pref.periodic_usage_stats_uuid_prefix = uid_prefix;
@@ -354,18 +338,13 @@ JsonNode *remmina_stats_get_user_env()
 
 	gchar *language;
 
-	language = setlocale (LC_ALL, NULL);
+	language = remmina_utils_get_lang();
 
 	b = json_builder_new();
 	json_builder_begin_object(b);
 	json_builder_set_member_name(b, "language");
 
-	if (!language || language[0] == '\0') {
-		json_builder_add_null_value(b);
-
-	}else {
-		json_builder_add_string_value(b, language);
-	}
+	json_builder_add_string_value(b, language);
 
 	json_builder_end_object(b);
 	r = json_builder_get_root(b);
@@ -540,25 +519,24 @@ JsonNode *remmina_stats_get_indicator()
 		/** Remmina not compiled with -DWITH_APPINDICATOR=on */
 		json_builder_add_int_value(b, 0);
 #endif
-	} else {
-		/** StatusNotifier/Appindicator NOT supported by desktop */
-		json_builder_add_int_value(b, 0);
-		json_builder_set_member_name(b, "icon_is_active");
-		if (remmina_icon_is_available()) {
-			/** Remmina icon is active */
-			json_builder_add_int_value(b, 1);
-			json_builder_set_member_name(b, "appindicator_type");
+	}
+	/** StatusNotifier/Appindicator NOT supported by desktop */
+	json_builder_add_int_value(b, 0);
+	json_builder_set_member_name(b, "icon_is_active");
+	if (remmina_icon_is_available()) {
+		/** Remmina icon is active */
+		json_builder_add_int_value(b, 1);
+		json_builder_set_member_name(b, "appindicator_type");
 #ifdef HAVE_LIBAPPINDICATOR
-			/** libappindicator fallback to GtkStatusIcon/xembed"); */
-			json_builder_add_string_value(b, "AppIndicator on GtkStatusIcon/xembed");
+		/** libappindicator fallback to GtkStatusIcon/xembed"); */
+		json_builder_add_string_value(b, "AppIndicator on GtkStatusIcon/xembed");
 #else
-			/** Remmina fallback to GtkStatusIcon/xembed */
-			json_builder_add_string_value(b, "Remmina icon on GtkStatusIcon/xembed");
+		/** Remmina fallback to GtkStatusIcon/xembed */
+		json_builder_add_string_value(b, "Remmina icon on GtkStatusIcon/xembed");
 #endif
-		}else {
-			/** Remmina icon is NOT active */
-			json_builder_add_int_value(b, 0);
-		}
+	}else {
+		/** Remmina icon is NOT active */
+		json_builder_add_int_value(b, 0);
 	}
 	json_builder_end_object(b);
 	r = json_builder_get_root(b);
@@ -579,29 +557,32 @@ static void remmina_profiles_get_data(RemminaFile *remminafile, gpointer user_da
 	gint count = 0;
 	gpointer pcount, kpo;
 	gpointer pdate, kdo;
-	gchar *sday, *smonth, *syear;
-	gchar *dday, *dmonth, *dyear;
+	gchar *hday, *hmonth, *hyear;
+	gchar *pday, *pmonth, *pyear;
 
-	GDateTime *ds;          /** Source date -> from profile */
-	GDateTime *dd;          /** Destination date -> The date in the pdata structure */
+	GDateTime *prof_gdate;          /** Source date -> from profile */
+	GDateTime *pdata_gdate;          /** Destination date -> The date in the pdata structure */
 
 	struct ProfilesData* pdata;
 	pdata = (struct ProfilesData*)user_data;
 
 	pdata->protocol = remmina_file_get_string(remminafile, "protocol");
-	pdata->pdatestr = remmina_file_get_string(remminafile, "last_success");
+	//pdata->pdatestr = remmina_file_get_string(remminafile, "last_success");
+	const gchar *last_success = remmina_file_get_string(remminafile, "last_success");
+	g_debug("%s date %s", pdata->protocol, last_success);
 
-	ds = dd = NULL;
-	if (pdata->pdatestr && pdata->pdatestr[0] != '\0' && strlen(pdata->pdatestr) >= 6) {
-		dyear = g_strdup_printf("%.4s", pdata->pdatestr);
-		dmonth = g_strdup_printf("%.2s", pdata->pdatestr + 4);
-		dday = g_strdup_printf("%.2s", pdata->pdatestr + 6);
-		dd = g_date_time_new_local(g_ascii_strtoll(dyear, NULL, 0),
-				g_ascii_strtoll(dmonth, NULL, 0),
-				g_ascii_strtoll(dday, NULL, 0), 0, 0, 0.0);
-		g_free(dyear);
-		g_free(dmonth);
-		g_free(dday);
+	prof_gdate = pdata_gdate = NULL;
+	if (last_success && last_success[0] != '\0' && strlen(last_success) >= 6) {
+		pyear = g_strdup_printf("%.4s", last_success);
+		pmonth = g_strdup_printf("%.2s", last_success + 4);
+		pday = g_strdup_printf("%.2s", last_success + 6);
+		prof_gdate = g_date_time_new_local(
+				atoi(pyear),
+				atoi(pmonth),
+				atoi(pday), 0, 0, 0);
+		g_free(pyear);
+		g_free(pmonth);
+		g_free(pday);
 	}
 
 
@@ -616,33 +597,49 @@ static void remmina_profiles_get_data(RemminaFile *remminafile, gpointer user_da
 		pdate = NULL;
 		if (g_hash_table_lookup_extended(pdata->proto_date, pdata->protocol, &kdo, &pdate)) {
 
-			ds = NULL;
+			pdata_gdate = NULL;
 			if (pdate && strlen(pdate) >= 6) {
-				syear = g_strdup_printf("%.4s", (char*)pdate);
-				smonth = g_strdup_printf("%.2s", (char*)pdate + 4);
-				sday = g_strdup_printf("%.2s", (char*)pdate + 6);
-				ds = g_date_time_new_local(g_ascii_strtoll(syear, NULL, 0),
-						g_ascii_strtoll(smonth, NULL, 0),
-						g_ascii_strtoll(sday, NULL, 0), 0, 0, 0.0);
-				g_free(syear);
-				g_free(smonth);
-				g_free(sday);
+				pdata->pdatestr = g_strdup(pdate);
+				hyear = g_strdup_printf("%.4s", (char*)pdate);
+				hmonth = g_strdup_printf("%.2s", (char*)pdate + 4);
+				hday = g_strdup_printf("%.2s", (char*)pdate + 6);
+				pdata_gdate = g_date_time_new_local(
+						atoi(hyear),
+						atoi(hmonth),
+						atoi(hday), 0, 0, 0);
+				g_free(hyear);
+				g_free(hmonth);
+				g_free(hday);
 			}
 
-			/** When both date in the has and in the profile are valid we compare the date */
-			if (ds && dd) {
-				gint res = g_date_time_compare( ds, dd );
+			/** When both date in the hash and in the profile are valid we compare the date */
+			if (prof_gdate != NULL && pdata_gdate != NULL ) {
+				g_debug("Comparing dates");
+				gint res = g_date_time_compare( pdata_gdate, prof_gdate );
 				/** If the date in the hash less than the date in the profile, we take the latter */
 				if (res < 0 ) {
+					g_debug("hash date is less than profile date. Replacing date in the hashtable");
+					g_hash_table_replace(pdata->proto_date, g_strdup(pdata->protocol), g_strdup(last_success));
+				} else {
+					g_debug("profile date is less than hash date. Replacing date in the hashtable");
 					g_hash_table_replace(pdata->proto_date, g_strdup(pdata->protocol), g_strdup(pdata->pdatestr));
 				}
+
 			}
-			/** If the date in the hash is NOT valid and the date in the profile is valid we keep the latter */
-			if (!ds && dd) {
+			/** If the date in the profile is NOT valid and the date in the hash is valid we keep the latter */
+			if (prof_gdate == NULL && pdata_gdate != NULL) {
+				g_debug("prof_gdate is NULL, replacing date in the hashtable");
 				g_hash_table_replace(pdata->proto_date, g_strdup(pdata->protocol), g_strdup(pdata->pdatestr));
 			}
+
+			/** If the date in the hash is NOT valid and the date in the profile is valid we keep the latter */
+			if (prof_gdate != NULL && pdata_gdate == NULL) {
+				g_debug("pdata_gdate is NULL, replacing date in the hashtable");
+				g_hash_table_replace(pdata->proto_date, g_strdup(pdata->protocol), g_strdup(last_success));
+			}
 			/** If both date are NULL, we insert NULL for that protocol */
-			if ((!ds && !dd) && pdata->pdatestr) {
+			if ((prof_gdate == NULL && pdata_gdate == NULL) && pdata->pdatestr) {
+				g_debug("All dates are NULL, replacing date in the hashtable");
 				g_hash_table_replace(pdata->proto_date, g_strdup(pdata->protocol), NULL);
 			}
 		}else {
@@ -656,10 +653,11 @@ static void remmina_profiles_get_data(RemminaFile *remminafile, gpointer user_da
 			}
 		}
 	}
-	if (dd)
-		g_date_time_unref(dd);
-	if (ds)
-		g_date_time_unref(ds);
+	g_debug("pdata set to %s protocol with last_success to %s",  pdata->protocol, pdata->pdatestr);
+	if (pdata_gdate)
+		g_date_time_unref(pdata_gdate);
+	if (prof_gdate)
+		g_date_time_unref(prof_gdate);
 }
 
 /**
@@ -717,6 +715,7 @@ JsonNode *remmina_stats_get_profiles()
 	profiles_count = remmina_file_manager_iterate(
 		(GFunc)remmina_profiles_get_data,
 		(gpointer)pdata);
+	g_debug("Number of profiles: %d", profiles_count);
 
 	json_builder_add_int_value(b, profiles_count);
 
@@ -729,9 +728,11 @@ JsonNode *remmina_stats_get_profiles()
 	g_hash_table_iter_init(&pdateiter, pdata->proto_date);
 	while (g_hash_table_iter_next(&pdateiter, &pdatekey, &pdatevalue)) {
 		s = g_strdup_printf("DATE_%s", (gchar*)pdatekey);
+		g_debug("Protocol date label: %s", s);
 		json_builder_set_member_name(b, s);
 		g_free(s);
 		json_builder_add_string_value(b, (gchar*)pdatevalue);
+		g_debug("Protocol date: %s", (gchar*)pdatevalue);
 	}
 
 	json_builder_end_object(b);
@@ -748,6 +749,97 @@ JsonNode *remmina_stats_get_profiles()
 	return r;
 }
 
+/**
+ * Add a json member ACTIVESECRETPLUGIN which shows the current secret plugin in use by remmina.
+ *
+ * @return a Json Node structure containg the secret plugin in use
+ *
+ */
+JsonNode *remmina_stats_get_secret_plugin()
+{
+	TRACE_CALL(__func__);
+
+	JsonBuilder *b;
+	JsonNode *r;
+	RemminaSecretPlugin *secret_plugin;
+	secret_plugin = remmina_plugin_manager_get_secret_plugin();
+
+	b = json_builder_new();
+	json_builder_begin_object(b);
+
+	if (secret_plugin && secret_plugin->is_service_available) {
+		json_builder_set_member_name(b, "plugin_name");
+		json_builder_add_string_value(b, secret_plugin->name);
+	}
+	json_builder_end_object(b);
+	r = json_builder_get_root(b);
+	g_object_unref(b);
+
+	return r;
+}
+
+/**
+ * Add a json member HASMASTERPASSWORD which shows the status of the master password.
+ *
+ * @return a Json Node structure containg the status of the master password
+ *
+ */
+JsonNode *remmina_stats_get_master_password_status()
+{
+	TRACE_CALL(__func__);
+
+	JsonBuilder *b;
+	JsonNode *r;
+
+	b = json_builder_new();
+	json_builder_begin_object(b);
+
+	json_builder_set_member_name(b, "master_password_status");
+	if (remmina_pref_get_boolean("use_master_password")) {
+		json_builder_add_string_value(b, "ON");
+	} else {
+		json_builder_add_string_value(b, "OFF");
+	}
+
+	json_builder_end_object(b);
+	r = json_builder_get_root(b);
+	g_object_unref(b);
+
+	return r;
+}
+
+/**
+ * Add a json member KIOSK which shows the status of the kiosk.
+ *
+ * @return a Json Node structure containg the status of the master password
+ *
+ */
+JsonNode *remmina_stats_get_kiosk_mode()
+{
+	TRACE_CALL(__func__);
+
+	JsonBuilder *b;
+	JsonNode *r;
+
+	b = json_builder_new();
+	json_builder_begin_object(b);
+
+	json_builder_set_member_name(b, "kiosk_status");
+	if (!kioskmode && kioskmode == FALSE) {
+		json_builder_add_string_value(b, "OFF");
+	}else {
+		json_builder_add_string_value(b, "ON");
+	}
+
+	json_builder_end_object(b);
+	r = json_builder_get_root(b);
+	g_object_unref(b);
+
+	return r;
+}
+
+
+
 /**
  * Get all statistics in json format to send periodically to the PHP server.
  * The caller should free the returned buffer with g_free()
@@ -807,6 +899,19 @@ JsonNode *remmina_stats_get_all()
 	json_builder_set_member_name(b, "PROFILES");
 	json_builder_add_value(b, n);
 
+	n = remmina_stats_get_secret_plugin();
+	json_builder_set_member_name(b, "ACTIVESECRETPLUGIN");
+	json_builder_add_value(b, n);
+
+	n = remmina_stats_get_master_password_status();
+	json_builder_set_member_name(b, "HASMASTERPASSWORD");
+	json_builder_add_value(b, n);
+
+	n = remmina_stats_get_kiosk_mode();
+	json_builder_set_member_name(b, "KIOSK");
+	json_builder_add_value(b, n);
+
+
 	json_builder_end_object(b);
 	n = json_builder_get_root(b);
 	g_object_unref(b);
