@@ -37,6 +37,9 @@
 #include "config.h"
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <fcntl.h>
 #include "remmina/remmina_trace_calls.h"
 #include "remmina_sysinfo.h"
 
